#!/usr/bin/env python
#coding:utf-8
from toughradius.console import models
from sqlalchemy.orm import scoped_session, sessionmaker
from toughradius.radiusd import utils
from toughradius.radiusd import pyrad
from toughradius.radiusd.pyrad import dictionary
from toughradius.radiusd.pyrad.client import Client 
from toughradius.radiusd.pyrad import packet
from toughradius.tools.dbengine import get_engine
import threading
import os,sys
import six
import random
import datetime
import time
import socket

status_vars = {'start':1,'stop':2,'update':3,'on':7,'off':8}

# random_sleeps = (0.01,0.02,0.03,0.04,0.05)
random_sleeps = (1.1,1.2,1.3,0.6,0.1,0.2)

# random_stop_sleeps = [ random.randrange(128,1200) for n in range(36) ]


class Connect:
    def __init__(self, mkdb):
        self.conn = mkdb()

    def __enter__(self):
        return self.conn   

    def __exit__(self, exc_type, exc_value, exc_tb):
        self.conn.close()
        
class RadSession(object):
    
    base_id = 0

    def __init__(self,user,session_id=None):
        if session_id is None:
            self.session_id = self.gen_id()
        else:
            self.session_id = session_id
        self.user = user
        self.start_time = int(time.time())
        self.last_update = int(time.time())
        self.session_time = 0
        self.stop_time = 0
        self.input_total = 0
        self.output_total = 0
        self.input_pkts = 0
        self.output_pkts = 0
        
    def update(self):
        self.input_total += random.randrange(2**10, 2**20)
        self.output_total += random.randrange(2**10, 2**20)
        self.input_pkts = random.randrange(1024, 81920)
        self.output_pkts = random.randrange(1024, 81920)
        _now = int(time.time())
        self.session_time = _now - self.last_update
        self.last_update = _now
        
            
    def gen_id(self):
        if self.base_id >= 9999:
            self.base_id=0
        self.base_id += 1
        _num = str(self.base_id).zfill(4)
        return datetime.datetime.now().strftime("%Y%m%d%H%M%S") + _num
        

class RadClient(object):
    
    def __init__(self,config):
        auth_port = config.getint('radiusd','authport')
        acct_port = config.getint('radiusd','acctport')
        rad_addr = '127.0.0.1'
        if config.has_option('radiusd','host'):
            rad_addr = config.getint('radiusd','host')
        dictfile = os.path.join(os.path.split(utils.__file__)[0],'dicts/dictionary')
        _dict = dictionary.Dictionary(dictfile)
        secret = six.b('radtest')
        self.debug = config.getboolean('DEFAULT','debug')
        self.radcli = Client(rad_addr,auth_port,acct_port,secret,_dict)
        self.radius_sessions = {}
        self.user_sessions = {}

    def send_pkt(self,req):
        try:
            print "Sending a radius request"
            
            if self.debug:
                req.source = ('127.0.0.1',0)
                print utils.format_packet_str(req)
            reply=self.radcli.SendPacket(req)
            print "recv a radius request"
            if self.debug:
                reply.source =  ('127.0.0.1',0)
                print utils.format_packet_str(reply)
            return reply
        except pyrad.client.Timeout:
            print "RADIUS server does not reply"
        except socket.error, error:
            print "Network error: " + error[1]

    def send_auth(self,user,pwd):
        req = self.radcli.CreateAuthPacket(
            code=pyrad.packet.AccessRequest,
            User_Name=user
        )
        req['User-Password'] = req.PwCrypt(pwd)
        req["NAS-IP-Address"]     = "192.168.88.10"
        req["Framed-IP-Address"]     = "192.168.88.11"
        req["NAS-Port"]           = 0
        req["Service-Type"]       = "Login-User"
        req["NAS-Identifier"]     = "radtest"
        req["Called-Station-Id"]  = "00-04-5F-00-0F-D1"
        req["Calling-Station-Id"] = "00-01-24-80-B3-9C"
        req['NAS-Port-Id'] = '3/0/1:0.0'

        return self.send_pkt(req)
            
    def send_acct_start(self,user):
        _session = RadSession(user)
        req=self.radcli.CreateAcctPacket(User_Name=user)
        print "Sending accounting start packet"
        req["NAS-IP-Address"]="192.168.88.10"
        req["Framed-IP-Address"] = "192.168.88.11"
        req["NAS-Port"]=0
        req["NAS-Identifier"]="radtest"
        req["Called-Station-Id"]="00-04-5F-00-0F-D1"
        req["Calling-Station-Id"]="00-01-24-80-B3-9C"
        req["Acct-Status-Type"]="Start"
        req['Acct-Session-Id'] = _session.session_id
        reply = self.send_pkt(req)
        self.radius_sessions[_session.session_id] = _session
        if user not in self.user_sessions:
            self.user_sessions[user] = []
        self.user_sessions[user].append(_session.session_id)
        return reply

    def send_acct_update(self,session):
        session.update()
        req=self.radcli.CreateAcctPacket(User_Name=session.user)
        print "Sending accounting update packet"
        req["NAS-IP-Address"]="192.168.88.10"
        req["NAS-Port"]=0
        req["NAS-Identifier"]="radtest"
        req["Framed-IP-Address"] = "192.168.88.11"
        req["Called-Station-Id"]="00-04-5F-00-0F-D1"
        req["Calling-Station-Id"]="00-01-24-80-B3-9C"
        req["Acct-Status-Type"]="Alive"
        req["Acct-Input-Octets"] = session.input_total
        req["Acct-Output-Octets"] = session.output_total
        req['Acct-Input-Packets'] = session.input_pkts
        req['Acct-Output-Packets'] = session.output_total
        req["Acct-Session-Time"] = session.session_time
        req['Acct-Session-Id'] = session.session_id
        return self.send_pkt(req)
        
    def send_acct_stop(self,session):
        session.update()
        req=self.radcli.CreateAcctPacket(User_Name=session.user)
        print "Sending accounting stop packet"
        req["NAS-IP-Address"]="192.168.88.10"
        req["NAS-Port"]=0
        req["NAS-Identifier"]="radtest"
        req["Framed-IP-Address"] = "192.168.88.11"
        req["Called-Station-Id"]="00-04-5F-00-0F-D1"
        req["Calling-Station-Id"]="00-01-24-80-B3-9C"
        req["Acct-Status-Type"]="stop"
        req["Acct-Input-Octets"] = session.input_total
        req["Acct-Output-Octets"] = session.output_total
        req['Acct-Input-Packets'] = session.input_pkts
        req['Acct-Output-Packets'] = session.output_total
        req["Acct-Session-Time"] = session.session_time
        req['Acct-Session-Id'] = session.session_id
        reply = self.send_pkt(req)
        del self.radius_sessions[session_id]
        if session.user not in self.user_sessions:
            self.user_sessions[sessiuon.user] = []
        self.user_sessions[session.user].remove(session.session_id)
        return reply

class Tester(object):
    
    def __init__(self,config):
        self.config = config
        self.mkdb = scoped_session(sessionmaker(
            bind=get_engine(config), autocommit=False, autoflush=True
        ))
        self.load_users()
        self.radcli = RadClient(config)
        self.running = False
        self.threads = []
        utils.aescipher.setup(config.get('DEFAULT','secret'))
        
    def load_users(self):
        with Connect(self.mkdb) as db:
            self.accounts = db.query(models.SlcRadAccount).all()
    
    def do_start(self):
        while self.running:
            user = random.choice(self.accounts)
            reply = self.radcli.send_auth(user.account_number,utils.decrypt(user.password))
            if reply.code == packet.AccessAccept:
                reply = self.radcli.send_acct_start(user.account_number)
            
            time.sleep(random.choice(random_sleeps))
            
    def do_update(self):
        while self.running:
            items = self.radcli.radius_sessions.values()
            if len(items) > 0:
                session = random.choice(items)
                reply = self.radcli.send_acct_update(session)
            time.sleep(random.choice(random_sleeps))
            
    def do_stop(self):
        while self.running:
            items = self.radcli.radius_sessions.values()
            if len(items) > 0:
                session = random.choice(items)
                reply = self.radcli.send_acct_stop(session)
            time.sleep(random.choice(random_sleeps))

    def start(self):
        self.running = True
        self.threads.append(threading.Thread(target=self.do_start))
        self.threads.append(threading.Thread(target=self.do_update))
        self.threads.append(threading.Thread(target=self.do_stop))
        
        # 启动线程
        for t in self.threads:
            t.start()
            
        while 1:
            if raw_input('type q to exit:') == 'q':
                self.running = False
                sys.exit(0)


            

        
            
     
            
      
    
    
        
    


#!/usr/bin/env python
#coding:utf-8
import sqlalchemy
import warnings
warnings.simplefilter('ignore', sqlalchemy.exc.SAWarning)
from sqlalchemy import *
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relation
from sqlalchemy.orm import scoped_session, sessionmaker
from hashlib import md5
from toughradius.console.libs import utils
import functools

DeclarativeBase = declarative_base()


def get_metadata(db_engine):
    global DeclarativeBase
    metadata = DeclarativeBase.metadata
    metadata.bind = db_engine
    return metadata

class SlcNode(DeclarativeBase):
    """区域表"""
    __tablename__ = 'slc_node'

    __table_args__ = {}

    #column definitions
    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"区域编号")
    node_name = Column(u'node_name', Unicode(length=32), nullable=False,doc=u"区域名")
    node_desc = Column(u'node_desc', Unicode(length=64), nullable=False,doc=u"区域描述")

    #relation definitions

class SlcOperator(DeclarativeBase):
    """操作员表 操作员类型 0 系统管理员 1 普通操作员"""
    __tablename__ = 'slc_operator'

    __table_args__ = {}

    #column definitions
    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"操作员id")
    operator_type = Column('operator_type', INTEGER(), nullable=False,doc=u"操作员类型")
    operator_name = Column(u'operator_name', Unicode(32), nullable=False,doc=u"操作员名称")
    operator_pass = Column(u'operator_pass', Unicode(length=128), nullable=False,doc=u"操作员密码")
    operator_status = Column(u'operator_status', INTEGER(), nullable=False,doc=u"操作员状态,0/1")
    operator_desc = Column(u'operator_desc', Unicode(255), nullable=False,doc=u"操作员描述")
    
class SlcOperatorNodes(DeclarativeBase):
    """操作员表关联区域"""
    __tablename__ = 'slc_operator_nodes'

    __table_args__ = {}

    #column definitions
    operator_name = Column(u'operator_name', Unicode(32),primary_key=True,nullable=False,doc=u"操作员名称")
    node_name = Column(u'node_name', Unicode(32), primary_key=True,nullable=False,doc=u"区域名称")

class SlcOperatorRule(DeclarativeBase):
    """操作员权限表"""
    __tablename__ = 'slc_operator_rule'

    __table_args__ = {}
    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"权限id")
    operator_name = Column(u'operator_name', Unicode(32), nullable=False,doc=u"操作员名称")
    rule_path = Column(u'rule_path', Unicode(128), nullable=False,doc=u"权限URL")
    rule_name = Column(u'rule_name', Unicode(128), nullable=False,doc=u"权限名称")
    rule_category = Column(u'rule_category', Unicode(128), nullable=False,doc=u"权限分类")


class SlcParam(DeclarativeBase):
    """系统参数表  <radiusd default table>"""
    __tablename__ = 'slc_param'

    __table_args__ = {}

    #column definitions
    param_name = Column(u'param_name', Unicode(length=64), primary_key=True, nullable=False,doc=u"参数名")
    param_value = Column(u'param_value', Unicode(length=255), nullable=False,doc=u"参数值")
    param_desc = Column(u'param_desc', Unicode(length=255),doc=u"参数描述")

    #relation definitions

class SlcRadBas(DeclarativeBase):
    """BAS设备表 <radiusd default table>"""
    __tablename__ = 'slc_rad_bas'

    __table_args__ = {}

    #column definitions
    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"设备id")
    vendor_id = Column(u'vendor_id', Unicode(length=32), nullable=False,doc=u"厂商标识")
    ip_addr = Column(u'ip_addr', Unicode(length=15), nullable=False,doc=u"IP地址")
    bas_name = Column(u'bas_name', Unicode(length=64), nullable=False,doc=u"bas名称")
    bas_secret = Column(u'bas_secret', Unicode(length=64), nullable=False,doc=u"共享密钥")
    coa_port = Column(u'coa_port', INTEGER(), nullable=False,doc=u"CoA端口")
    time_type = Column(u'time_type', SMALLINT(), nullable=False,doc=u"时区类型")

    #relation definitions



class SlcRadRoster(DeclarativeBase):
    """黑白名单 0 白名单 1 黑名单 <radiusd default table>"""
    __tablename__ = 'slc_rad_roster'

    __table_args__ = {}

    #column definitions
    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"黑白名单id")
    mac_addr = Column('mac_addr', Unicode(length=17), nullable=False,doc=u"mac地址")
    begin_time = Column('begin_time', Unicode(length=19), nullable=False,doc=u"生效开始时间")
    end_time = Column('end_time', Unicode(length=19), nullable=False,doc=u"生效结束时间")
    roster_type = Column('roster_type', SMALLINT(), nullable=False,doc=u"黑白名单类型")


class SlcMember(DeclarativeBase):
    """用户信息表"""
    __tablename__ = 'slc_member'

    __table_args__ = {}

    member_id = Column('member_id', INTEGER(),
        Sequence('member_id_seq', start=100001, increment=1),
        primary_key=True,nullable=False,doc=u"用户id")
    node_id = Column('node_id', INTEGER(), nullable=False,doc=u"区域id")
    member_name = Column('member_name', Unicode(length=64), nullable=False,doc=u"用户登录名")
    password = Column('password', Unicode(length=128), nullable=False,doc=u"用户登录密码")
    realname = Column('realname', Unicode(length=64), nullable=False,doc=u"")
    idcard = Column('idcard', Unicode(length=32),doc=u"用户证件号码")
    sex = Column('sex', SMALLINT(), nullable=True,doc=u"用户性别0/1")
    age = Column('age', INTEGER(), nullable=True,doc=u"用户年龄")
    email = Column('email', Unicode(length=255), nullable=True,doc=u"用户邮箱")
    email_active = Column('email_active', SMALLINT(), default=0,doc=u"用户邮箱激活状态")
    active_code =  Column('active_code', Unicode(length=32), nullable=False,doc=u"邮箱激活码")
    mobile = Column('mobile', Unicode(length=16), nullable=True,doc=u"用户手机")
    mobile_active = Column('mobile_active', SMALLINT(), default=0,doc=u"用户手机绑定状态")
    address = Column('address', Unicode(length=255), nullable=True,doc=u"用户地址")
    create_time = Column('create_time', Unicode(length=19), nullable=False,doc=u"创建时间")
    update_time = Column('update_time', Unicode(length=19), nullable=False,doc=u"更新时间")


class SlcMemberOrder(DeclarativeBase):
    """
    订购信息表(交易记录)
    pay_status交易支付状态：0-未支付，1-已支付，2-已取消
    """
    __tablename__ = 'slc_member_order'

    __table_args__ = {}

    order_id = Column('order_id', Unicode(length=32),primary_key=True,nullable=False,doc=u"订单id")
    member_id = Column('member_id', INTEGER(),nullable=False,doc=u"用户id")
    product_id = Column('product_id', INTEGER(),nullable=False,doc=u"资费id")
    account_number = Column('account_number', Unicode(length=32),nullable=False,doc=u"上网账号")
    order_fee = Column('order_fee', INTEGER(), nullable=False,doc=u"订单费用")
    actual_fee = Column('actual_fee', INTEGER(), nullable=False,doc=u"实缴费用")
    pay_status = Column('pay_status', INTEGER(), nullable=False,doc=u"支付状态")
    accept_id = Column('accept_id', INTEGER(),nullable=False,doc=u"受理id")
    order_source = Column('order_source', Unicode(length=64), nullable=False,doc=u"订单来源")
    order_desc = Column('order_desc', Unicode(length=255),doc=u"订单描述")
    create_time = Column('create_time', Unicode(length=19), nullable=False,doc=u"交易时间")


class SlcRechargerCard(DeclarativeBase):
    """
    充值卡表
    批次号：batch_no，以年月开始紧跟顺序号，如20150201
    卡类型 0 资费卡   1 余额卡
    状态 card_status 0 未激活 1 已激活 2 已使用 3 已回收
    """
    __tablename__ = 'slc_recharge_card'

    __table_args__ = {}

    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"充值卡id")
    batch_no = Column('batch_no', INTEGER(), nullable=False,doc=u"批次号")
    card_number = Column('card_number', Unicode(length=16),nullable=False,unique=True,doc=u"充值卡号")
    card_passwd = Column('card_passwd', Unicode(length=128),nullable=False,doc=u"充值卡密码")
    card_type = Column('card_type', INTEGER(),nullable=False,doc=u"充值卡类型")
    card_status = Column('card_status', INTEGER(), nullable=False,doc=u"状态")
    product_id = Column('product_id', INTEGER(),nullable=True,doc=u"资费id")
    fee_value = Column('fee_value', INTEGER(), nullable=False,doc=u"充值卡面值-元")
    months = Column('months', INTEGER(),nullable=True,doc=u"授权月数")
    times = Column('times', INTEGER(),nullable=True,doc=u"授权时长(秒)")
    flows = Column('flows', INTEGER(),nullable=True,doc=u"授权流量(kb)")
    expire_date = Column('expire_date', Unicode(length=10), nullable=False,doc=u"过期时间- ####-##-##")
    create_time = Column('create_time', Unicode(length=19), nullable=False,doc=u"创建时间")

class SlcRechargeLog(DeclarativeBase):
    """
    充值记录
    """
    __tablename__ = 'slc_recharge_log'

    __table_args__ = {}

    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"日志id")
    card_number = Column('card_number', Unicode(length=16),nullable=False,doc=u"充值卡号")
    member_id = Column('member_id', INTEGER(),nullable=False,doc=u"用户id")
    account_number = Column('account_number', Unicode(length=32),nullable=False,doc=u"上网账号")
    recharge_status = Column('recharge_status', INTEGER(), nullable=False,doc=u"充值结果")
    recharge_time = Column('recharge_time', Unicode(length=19), nullable=False,doc=u"充值时间")


class SlcRadAccount(DeclarativeBase):
    """
    上网账号表，每个会员可以同时拥有多个上网账号
    account_number 为每个套餐对应的上网账号，每个上网账号全局唯一
    用户状态 0:"预定",1:"正常", 2:"停机" , 3:"销户", 4:"到期"
    <radiusd default table>
    """

    __tablename__ = 'slc_rad_account'

    __table_args__ = {}

    account_number = Column('account_number', Unicode(length=32),primary_key=True,nullable=False,doc=u"上网账号")
    member_id = Column('member_id', INTEGER(),nullable=False,doc=u"用户id")
    product_id = Column('product_id', INTEGER(),nullable=False,doc=u"资费id")
    group_id = Column('group_id', INTEGER(),doc=u"用户组id")
    password = Column('password', Unicode(length=128), nullable=False,doc=u"上网密码")
    status = Column('status', INTEGER(), nullable=False,doc=u"用户状态")
    install_address = Column('install_address', Unicode(length=128), nullable=False,doc=u"装机地址")
    balance = Column('balance', INTEGER(), nullable=False,doc=u"用户余额-分")
    time_length = Column('time_length', INTEGER(), nullable=False,default=0,doc=u"用户时长-秒")
    flow_length = Column('flow_length', INTEGER(), nullable=False,default=0,doc=u"用户流量-kb")
    expire_date = Column('expire_date', Unicode(length=10), nullable=False,doc=u"过期时间- ####-##-##")
    user_concur_number = Column('user_concur_number', INTEGER(), nullable=False,doc=u"用户并发数")
    bind_mac = Column('bind_mac', SMALLINT(), nullable=False,doc=u"是否绑定mac")
    bind_vlan = Column('bind_vlan', SMALLINT(), nullable=False,doc=u"是否绑定vlan")
    mac_addr = Column('mac_addr', Unicode(length=17),doc=u"mac地址")
    vlan_id = Column('vlan_id', INTEGER(),doc=u"内层vlan")
    vlan_id2 = Column('vlan_id2', INTEGER(),doc=u"外层vlan")
    ip_address = Column('ip_address', Unicode(length=15),doc=u"静态IP地址")
    last_pause = Column('last_pause', Unicode(length=19),doc=u"最后停机时间")
    create_time = Column('create_time', Unicode(length=19), nullable=False,doc=u"创建时间")
    update_time = Column('update_time', Unicode(length=19), nullable=False,doc=u"更新时间")

class SlcRadAccountAttr(DeclarativeBase):
    """上网账号扩展策略属性表"""
    __tablename__ = 'slc_rad_account_attr'
    __table_args__ = {}

    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"属性id")
    account_number = Column('account_number', Unicode(length=32),nullable=False,doc=u"上网账号")
    attr_name = Column(u'attr_name', Unicode(length=255), nullable=False,doc=u"属性名")
    attr_value = Column(u'attr_value', Unicode(length=255), nullable=False,doc=u"属性值")
    attr_desc = Column(u'attr_desc', Unicode(length=255),doc=u"属性描述")

class SlcRadProduct(DeclarativeBase):
    '''
    资费信息表 <radiusd default table>
    资费类型 product_policy 0 预付费包月 1 预付费时长 2 买断包月 3 买断时长 4 预付费流量 5 买断流量
    销售状态 product_status 0 正常 1 停用 资费停用后不允许再订购
    '''
    __tablename__ = 'slc_rad_product'

    __table_args__ = {}

    id = Column('id', INTEGER(),primary_key=True,autoincrement=1,nullable=False,doc=u"资费id")
    product_name = Column('product_name', Unicode(length=64), nullable=False,doc=u"资费名称")
    product_policy = Column('product_policy', INTEGER(), nullable=False,doc=u"资费策略")
    product_status = Column('product_status', SMALLINT(), nullable=False,doc=u"资费状态")
    bind_mac = Column('bind_mac', SMALLINT(), nullable=False,doc=u"是否绑定mac")
    bind_vlan = Column('bind_vlan', SMALLINT(), nullable=False,doc=u"是否绑定vlan")
    concur_number = Column('concur_number', INTEGER(), nullable=False,doc=u"并发数")
    fee_period = Column('fee_period', Unicode(length=11),doc=u"开放认证时段")
    fee_months = Column('fee_months', INTEGER(),doc=u"买断授权月数")
    fee_times = Column('fee_times', INTEGER(),doc=u"买断时长(秒)")
    fee_flows = Column('fee_flows', INTEGER(),doc=u"买断流量(kb)")
    fee_price = Column('fee_price', INTEGER(),nullable=False,doc=u"资费价格")
    input_max_limit = Column('input_max_limit', INTEGER(), nullable=False,doc=u"上行速率")
    output_max_limit = Column('output_max_limit', INTEGER(), nullable=False,doc=u"下行速率")
    create_time = Column('create_time', Unicode(length=19), nullable=False,doc=u"创建时间")
    update_time = Column('update_time', Unicode(length=19), nullable=False,doc=u"更新时间")

class SlcRadProductAttr(DeclarativeBase):
    '''资费扩展属性表 <radiusd default table>'''
    __tablename__ = 'slc_rad_product_attr'

    __table_args__ = {}

    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"属性id")
    product_id = Column('product_id', INTEGER(),nullable=False,doc=u"资费id")
    attr_name = Column(u'attr_name', Unicode(length=255), nullable=False,doc=u"属性名")
    attr_value = Column(u'attr_value', Unicode(length=255), nullable=False,doc=u"属性值")
    attr_desc = Column(u'attr_desc', Unicode(length=255),doc=u"属性描述")

class SlcRadBilling(DeclarativeBase):
    """计费信息表 is_deduct 0 未扣费 1 已扣费 <radiusd default table>"""
    __tablename__ = 'slc_rad_billing'

    __table_args__ = { }

    #column definitions
    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"计费id")
    account_number = Column(u'account_number', Unicode(length=253), nullable=False,doc=u"上网账号")
    nas_addr = Column(u'nas_addr', Unicode(length=15), nullable=False,doc=u"bas地址")
    acct_session_id = Column(u'acct_session_id', Unicode(length=253), nullable=False,doc=u"会话id")
    acct_start_time = Column(u'acct_start_time', Unicode(length=19), nullable=False,doc=u"计费开始时间")
    acct_session_time = Column(u'acct_session_time', INTEGER(), nullable=False,doc=u"会话时长")
    input_total = Column(u'input_total', INTEGER(),doc=u"会话的上行流量（kb）")
    output_total = Column(u'output_total', INTEGER(),doc=u"会话的下行流量（kb）")
    acct_times = Column(u'acct_times', INTEGER(), nullable=False,doc=u"扣费时长(秒)")
    acct_flows = Column(u'acct_flows', INTEGER(), nullable=False,doc=u"扣费流量(kb)")
    acct_fee = Column(u'acct_fee', INTEGER(), nullable=False,doc=u"应扣费用")
    actual_fee = Column('actual_fee', INTEGER(), nullable=False,doc=u"实扣费用")
    balance = Column('balance', INTEGER(), nullable=False,doc=u"当前余额")
    time_length = Column('time_length', INTEGER(), nullable=False,default=0,doc=u"当前用户时长-秒")
    flow_length = Column('flow_length', INTEGER(), nullable=False,default=0,doc=u"当前用户流量-kb")
    is_deduct = Column(u'is_deduct', INTEGER(), nullable=False,doc=u"是否扣费")
    create_time = Column('create_time', Unicode(length=19), nullable=False,doc=u"计费时间")


class SlcRadTicket(DeclarativeBase):
    """上网日志表 <radiusd default table>"""
    __tablename__ = 'slc_rad_ticket'

    __table_args__ = { }

    #column definitions
    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"日志id")
    account_number = Column(u'account_number', Unicode(length=253), nullable=False,doc=u"上网账号")
    acct_input_gigawords = Column(u'acct_input_gigawords', INTEGER(),doc=u"会话的上行的字（4字节）的吉倍数")
    acct_output_gigawords = Column(u'acct_output_gigawords', INTEGER(),doc=u"会话的下行的字（4字节）的吉倍数")
    acct_input_octets = Column(u'acct_input_octets', INTEGER(),doc=u"会话的上行流量（字节数）")
    acct_output_octets = Column(u'acct_output_octets', INTEGER(),doc=u"会话的下行流量（字节数）")
    acct_input_packets = Column(u'acct_input_packets', INTEGER(),doc=u"会话的上行包数量")
    acct_output_packets = Column(u'acct_output_packets', INTEGER(),doc=u"会话的下行包数量")
    acct_session_id = Column(u'acct_session_id', Unicode(length=253), nullable=False,doc=u"会话id")
    acct_session_time = Column(u'acct_session_time', INTEGER(), nullable=False,doc=u"会话时长")
    acct_start_time = Column(u'acct_start_time', Unicode(length=19), nullable=False,doc=u"会话开始时间")
    acct_stop_time = Column(u'acct_stop_time', Unicode(length=19), nullable=False,doc=u"会话结束时间")
    acct_terminate_cause = Column(u'acct_terminate_cause',INTEGER(),doc=u"会话中止原因")
    mac_addr = Column(u'mac_addr', Unicode(length=128),doc=u"mac地址")
    calling_station_id =  Column(u'calling_station_id', Unicode(length=128),doc=u"用户接入物理信息")
    framed_netmask = Column(u'frame_id_netmask', Unicode(length=15),doc=u"地址掩码")
    framed_ipaddr = Column(u'framed_ipaddr', Unicode(length=15),doc=u"IP地址")
    nas_class = Column(u'nas_class', Unicode(length=253),doc=u"bas class")
    nas_addr = Column(u'nas_addr', Unicode(length=15), nullable=False,doc=u"bas地址")
    nas_port = Column(u'nas_port', Unicode(length=32),doc=u"接入端口")
    nas_port_id = Column(u'nas_port_id', Unicode(length=255),doc=u"接入端口物理信息")
    nas_port_type = Column(u'nas_port_type', INTEGER(),doc=u"接入端口类型")
    service_type = Column(u'service_type', INTEGER(),doc=u"接入服务类型")
    session_timeout = Column(u'session_timeout', INTEGER(),doc=u"会话超时时间")
    start_source = Column(u'start_source', INTEGER(), nullable=False,doc=u"会话开始来源")
    stop_source = Column(u'stop_source', INTEGER(), nullable=False,doc=u"会话中止来源")

    #relation definitions

class SlcRadOnline(DeclarativeBase):
    """用户在线信息表 <radiusd default table>"""
    __tablename__ = 'slc_rad_online'

    __table_args__ = {
        'mysql_engine' : 'MEMORY'
    }

    #column definitions
    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"在线id")
    account_number = Column(u'account_number', Unicode(length=32), nullable=False,doc=u"上网账号")
    nas_addr = Column(u'nas_addr', Unicode(length=32), nullable=False,doc=u"bas地址")
    acct_session_id = Column(u'acct_session_id', Unicode(length=64), nullable=False,doc=u"会话id")
    acct_start_time = Column(u'acct_start_time', Unicode(length=19), nullable=False,doc=u"会话开始时间")
    framed_ipaddr = Column(u'framed_ipaddr', Unicode(length=32), nullable=False,doc=u"IP地址")
    mac_addr = Column(u'mac_addr', Unicode(length=32), nullable=False,doc=u"mac地址")
    nas_port_id = Column(u'nas_port_id', Unicode(length=255), nullable=False,doc=u"接入端口物理信息")
    billing_times = Column(u'billing_times', INTEGER(), nullable=False,doc=u"已记账时间")
    input_total = Column(u'input_total', INTEGER(),doc=u"上行流量（kb）")
    output_total = Column(u'output_total', INTEGER(),doc=u"下行流量（kb）")
    start_source = Column(u'start_source', SMALLINT(), nullable=False,doc=u"记账开始来源")

class SlcRadOnlineStat(DeclarativeBase):
    """用户在线统计表 """
    __tablename__ = 'slc_rad_online_stat'

    __table_args__ = {}

    #column definitions
    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"id")
    node_id = Column('node_id', INTEGER(),nullable=False,doc=u"区域id")
    stat_time = Column(u'stat_time', INTEGER(),  nullable=False,doc=u"统计时间")
    total = Column(u'total', INTEGER(),doc=u"在线数")

    #relation definitions

class SlcRadFlowStat(DeclarativeBase):
    """用户在线统计表 """
    __tablename__ = 'slc_rad_flow_stat'

    __table_args__ = {}

    #column definitions
    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"id")
    node_id = Column('node_id', INTEGER(),nullable=False,doc=u"区域id")
    stat_time = Column(u'stat_time', INTEGER(),  nullable=False,doc=u"统计时间")
    input_total = Column(u'input_total', INTEGER(),doc=u"上行流量（kb）")
    output_total = Column(u'output_total', INTEGER(),doc=u"下行流量（kb）")

    #relation definitions

class SlcRadAcceptLog(DeclarativeBase):
    '''
    业务受理日志表
    open:开户 pause:停机 resume:复机 cancel:销户 next:续费 charge:充值
    '''
    __tablename__ = 'slc_rad_accept_log'
    __table_args__ = {}

    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"日志id")
    accept_type = Column(u'accept_type', Unicode(length=16), nullable=False,doc=u"受理类型")
    accept_desc = Column(u'accept_desc', Unicode(length=512),doc=u"受理描述")
    account_number = Column(u'account_number', Unicode(length=32), nullable=False,doc=u"上网账号")
    operator_name = Column(u'operator_name', Unicode(32),doc=u"操作员名")
    accept_source = Column(u'accept_source', Unicode(length=128),doc=u"受理渠道来源")
    accept_time = Column(u'accept_time', Unicode(length=19), nullable=False,doc=u"受理时间")

class SlcRadOperateLog(DeclarativeBase):
    """操作日志表"""
    __tablename__ = 'slc_rad_operate_log'

    __table_args__ = {}

    #column definitions
    id = Column(u'id', INTEGER(), primary_key=True, nullable=False,doc=u"日志id")
    operator_name = Column(u'operator_name', Unicode(32), nullable=False,doc=u"操作员名称")
    operate_ip = Column(u'operate_ip', Unicode(length=128),doc=u"操作员ip")
    operate_time = Column(u'operate_time', Unicode(length=19), nullable=False,doc=u"操作时间")
    operate_desc = Column(u'operate_desc', Unicode(length=1024),doc=u"操作描述")

def init_db(db):
    node = SlcNode()
    node.id = 1
    node.node_name = 'default'
    node.node_desc = u'测试区域'
    db.add(node)

    params = [
        ('system_name',u'管理系统名称',u'ToughRADIUS管理控制台'),
        ('customer_system_name',u'自助服务系统名称',u'ToughRADIUS自助服务中心'),
        ('customer_system_url',u"自助服务系统地址",u"http://forum.toughradius.net"),
        ('customer_must_active',u"激活邮箱才能自助开户充值(0:否|1:是)",u"0"),
        ('radiusd_address',u'Radius服务IP地址',u'127.0.0.1'),
        ('radiusd_admin_port',u'Radius服务管理端口',u'1815'),
        ('is_debug',u'DEBUG模式',u'0'),
        ('weixin_qrcode',u'微信公众号二维码图片(宽度230px)',u'http://img.toughradius.net/toughforum/jamiesun/1421820686.jpg!230'),
        ('service_phone',u'客户服务电话',u'000000'),
        ('service_qq',u'客户服务QQ号码',u'000000'),
        ('rcard_order_url',u'充值卡订购网站地址',u'http://www.tmall.com'),
        ('portal_secret',u'portal登陆密钥', u'abcdefg123456'),
        ('expire_notify_days','到期提醒提前天数',u'7'),
        ('expire_notify_tpl','到期提醒邮件模板',u'账号到期通知\n尊敬的会员您好:\n您的账号#account#即将在#expire#到期，请及时续费！'),
        ('expire_notify_url',u'到期通知url',u'http://your_notify_url?account={account}&expire={expire}&email={email}&mobile={mobile}'),
        ('expire_addrpool',u'到期提醒下发地址池',u'expire'),
        ('expire_session_timeout',u'到期用户下发最大会话时长(秒)',u'120'),
        ('smtp_server',u'SMTP服务器地址',u'smtp.mailgun.org'),
        ('smtp_user',u'SMTP用户名',u'service@toughradius.org'),
        ('smtp_pwd',u'SMTP密码',u'service2015'),
        ('smtp_sender',u'SMTP发送人名称',u'运营中心'),
        ('acct_interim_intelval',u'Radius记账间隔(秒)',u'120'),
        ('max_session_timeout',u'Radius最大会话时长(秒)',u'86400'),
        ('reject_delay',u'拒绝延迟时间(秒)(0-9)','0')
    ]

    for p in params:
        param = SlcParam()
        param.param_name = p[0]
        param.param_desc = p[1]
        param.param_value = p[2]
        db.add(param)

    opr = SlcOperator()
    opr.id = 1
    opr.operator_name = u'admin'
    opr.operator_type = 0
    opr.operator_pass = md5('root').hexdigest()
    opr.operator_desc = 'admin'
    opr.operator_status = 0
    db.add(opr)

    db.commit()
    db.close()

def update(db_engine):
    print 'starting update database...'
    metadata = get_metadata(db_engine)
    metadata.drop_all(db_engine)
    metadata.create_all(db_engine)
    print 'update database done'
    db = scoped_session(sessionmaker(bind=db_engine, autocommit=False, autoflush=True))()
    init_db(db)

if __name__ == '__main__':
    update()



#!/usr/bin/env python
#coding:utf-8
import sys,os
import time
import smtplib
from email.mime.text import MIMEText
from email.Header import Header

class Mail(object):
    
    def setup(self,server=None,user=None,pwd=None,fromaddr=None,sender=None):
        self.server = server
        self.user = user
        self.pwd = pwd
        self.fromaddr = fromaddr
        self.sender = sender

    def sendmail(self,mailto,topic,content):
        if not mailto or not topic:return
        #print 'mailto',mailto,topic,content
        topic = topic.replace("\n","<br>")
        content = content.replace("\n","<br>")
        mail = MIMEText(content, 'html', 'utf-8')
        mail['Subject'] = Header("[Notify]:%s"%topic,'utf-8')
        mail['From'] = Header("%s <%s>"%(self.fromaddr[:self.fromaddr.find('@')],self.fromaddr),'utf-8')
        mail['To'] = mailto
        mail["Accept-Language"]="zh-CN"
        mail["Accept-Charset"]="ISO-8859-1,utf-8"
        if '@toughradius.org' in self.fromaddr:
            mail['X-Mailgun-SFlag'] = 'yes'
            mail['X-Mailgun-SScore'] = 'yes'
        try:
            serv = smtplib.SMTP()
            # serv.set_debuglevel(True)
            serv.connect(self.server)
            if self.pwd and self.pwd not in ('no','none','anonymous'):
                serv.login(self.user,self.pwd)
            serv.sendmail(self.fromaddr, [mailto], mail.as_string())
            serv.quit()
            print "Successfully sent email to %s"%mailto
        except Exception,e:
            print "Error: unable to send email %s"%str(e)
            
mail = Mail()


#!/usr/bin/env python
#coding:utf-8
from bottle import Bottle
from bottle import request
from bottle import response
from bottle import redirect
from bottle import run as runserver
from bottle import static_file
from bottle import abort
from hashlib import md5
from tablib import Dataset
from toughradius.console.base import *
from toughradius.console.libs import utils
from toughradius.console.websock import websock
from toughradius.console import models
import bottle
from toughradius.console.admin import forms
import decimal
import datetime

__prefix__ = "/bus"

decimal.getcontext().prec = 11
decimal.getcontext().rounding = decimal.ROUND_UP

app = Bottle()
app.config['__prefix__'] = __prefix__
render = functools.partial(Render.render_app,app)


###############################################################################
# ajax query
###############################################################################

@app.get('/product/json',apply=auth_opr)
def product_get(db):
    node_id = request.params.get('node_id')
    if not node_id:return dict(code=1,data=[])
    items = db.query(models.SlcRadProduct).filter_by(node_id=node_id)
    return dict(
        code=0,
        data=[{'code': it.id,'name': it.product_name} for it in items]
    )
    
@app.get('/product/policy/get',apply=auth_opr)
def product_policy_get(db):
    product_id = request.params.get("product_id")
    product_policy = db.query(
        models.SlcRadProduct.product_policy
    ).filter_by(id = product_id).scalar()
    return dict(
        code=0,
        data={'id': product_id,'policy': product_policy}
    )


@app.post('/opencalc',apply=auth_opr)
def opencalc(db):
    months = request.params.get('months',0)
    product_id = request.params.get("product_id")
    old_expire = request.params.get("old_expire")
    product = db.query(models.SlcRadProduct).get(product_id)
    # 预付费时长，预付费流量，
    if product.product_policy in (PPTimes,PPFlow):
        return dict(code=0,data=dict(policy=product.product_policy,fee_value=0,expire_date="3000-12-30"))
    # 买断时长 买断流量
    elif product.product_policy in (BOTimes,BOFlows):
        fee_value = utils.fen2yuan(product.fee_price)
        return dict(code=0,data=dict(policy=product.product_policy,fee_value=fee_value,expire_date="3000-12-30"))
    # 预付费包月 
    elif product.product_policy == PPMonth:
        fee = decimal.Decimal(months) * decimal.Decimal(product.fee_price)
        fee_value = utils.fen2yuan(int(fee.to_integral_value()))
        start_expire = datetime.datetime.now()
        if old_expire:
            start_expire = datetime.datetime.strptime(old_expire,"%Y-%m-%d")
        expire_date = utils.add_months(start_expire,int(months))
        expire_date = expire_date.strftime( "%Y-%m-%d")
        return dict(code=0,data=dict(policy=product.product_policy,fee_value=fee_value,expire_date=expire_date))
    # 买断包月
    elif product.product_policy == BOMonth:
        start_expire = datetime.datetime.now()
        if old_expire:
            start_expire = datetime.datetime.strptime(old_expire,"%Y-%m-%d")
        fee_value = utils.fen2yuan(product.fee_price)
        expire_date = utils.add_months(start_expire,product.fee_months)
        expire_date = expire_date.strftime( "%Y-%m-%d")
        return dict(code=0,data=dict(policy=product.product_policy,fee_value=fee_value,expire_date=expire_date))

###############################################################################
# member query
###############################################################################

@app.route('/member',apply=auth_opr,method=['GET','POST'])
@app.post('/member/export',apply=auth_opr)
def member_query(db):
    node_id = request.params.get('node_id')
    realname = request.params.get('realname')
    idcard = request.params.get('idcard')
    mobile = request.params.get('mobile')
    user_name = request.params.get('user_name')
    status = request.params.get('status')
    product_id = request.params.get('product_id')
    address = request.params.get('address')
    expire_days = request.params.get('expire_days')
    opr_nodes = get_opr_nodes(db)
    _query = db.query(
            models.SlcMember,
            models.SlcRadAccount,
            models.SlcRadProduct.product_name,
            models.SlcNode.node_desc
        ).filter(
            models.SlcRadProduct.id == models.SlcRadAccount.product_id,
            models.SlcMember.member_id == models.SlcRadAccount.member_id,
            models.SlcNode.id == models.SlcMember.node_id
        )
    if idcard:
        _query = _query.filter(models.SlcMember.idcard==idcard)
    if mobile:
        _query = _query.filter(models.SlcMember.mobile==mobile)
    if node_id:
        _query = _query.filter(models.SlcMember.node_id == node_id)
    else:
        _query = _query.filter(models.SlcMember.node_id.in_([i.id for i in opr_nodes]))
    if realname:
        _query = _query.filter(models.SlcMember.realname.like('%'+realname+'%'))
    if user_name:
        _query = _query.filter(models.SlcRadAccount.account_number.like('%'+user_name+'%'))
    if status:
        _query = _query.filter(models.SlcRadAccount.status == status)
    if product_id:
        _query = _query.filter(models.SlcRadAccount.product_id==product_id)
    if address:
        _query = _query.filter(models.SlcMember.address.like('%'+address+'%'))
    if expire_days:
        _days = int(expire_days)
        td = datetime.timedelta(days=30)
        _now = datetime.datetime.now() 
        edate = (_now + td).strftime("%Y-%m-%d") 
        _query = _query.filter(models.SlcRadAccount.expire_date <= edate)
        _query = _query.filter(models.SlcRadAccount.expire_date >= _now.strftime("%Y-%m-%d"))
        

    if request.path == '/member':
        return render("bus_member_list", 
            page_data = get_page_data(_query),
            node_list=opr_nodes,
            products=db.query(models.SlcRadProduct),
            **request.params)
    elif request.path == "/member/export":
        data = Dataset()
        data.append((
            u'区域',u'姓名',u'证件号',u'邮箱', u'联系电话', u'地址',
            u'用户账号',u'密码',u'资费', u'过期时间', u'余额(元)',
            u'时长(小时)',u'流量(MB)',u'并发数',u'ip地址',u'状态',u'创建时间'
        ))
        for i,j,_product_name,_node_desc in _query:
            data.append((
                _node_desc,i.realname,i.idcard,i.email,i.mobile, i.address,
                j.account_number,utils.decrypt(j.password), _product_name, 
                j.expire_date,utils.fen2yuan(j.balance),
                utils.sec2hour(j.time_length),utils.kb2mb(j.flow_length),j.user_concur_number,j.ip_address,
                forms.user_state[j.status],j.create_time
            ))
        name = u"RADIUS-USER-" + datetime.datetime.now().strftime("%Y%m%d-%H%M%S") + ".xls"
        return export_file(name,data)

permit.add_route("/bus/member",u"用户信息管理",u"营业管理",is_menu=True,order=0)
permit.add_route("/bus/member/export",u"用户信息导出",u"营业管理",order=0.01)

@app.get('/member/detail',apply=auth_opr)
def member_detail(db):
    account_number = request.params.get('account_number')  
    user  = db.query(
        models.SlcMember.realname,
        models.SlcRadAccount.member_id,
        models.SlcRadAccount.account_number,
        models.SlcRadAccount.expire_date,
        models.SlcRadAccount.balance,
        models.SlcRadAccount.time_length,
        models.SlcRadAccount.flow_length,
        models.SlcRadAccount.user_concur_number,
        models.SlcRadAccount.status,
        models.SlcRadAccount.mac_addr,
        models.SlcRadAccount.vlan_id,
        models.SlcRadAccount.vlan_id2,
        models.SlcRadAccount.ip_address,
        models.SlcRadAccount.bind_mac,
        models.SlcRadAccount.bind_vlan,
        models.SlcRadAccount.ip_address,
        models.SlcRadAccount.install_address,
        models.SlcRadAccount.last_pause,
        models.SlcRadAccount.create_time,
        models.SlcRadProduct.product_name,
        models.SlcRadProduct.product_policy
    ).filter(
            models.SlcRadProduct.id == models.SlcRadAccount.product_id,
            models.SlcMember.member_id == models.SlcRadAccount.member_id,
            models.SlcRadAccount.account_number == account_number
    ).first()
    member = db.query(models.SlcMember).get(user.member_id)
    orders = db.query(
            models.SlcMemberOrder.order_id,
            models.SlcMemberOrder.order_id,
            models.SlcMemberOrder.product_id,
            models.SlcMemberOrder.account_number,
            models.SlcMemberOrder.order_fee,
            models.SlcMemberOrder.actual_fee,
            models.SlcMemberOrder.pay_status,
            models.SlcMemberOrder.create_time,
            models.SlcMemberOrder.order_desc,
            models.SlcRadProduct.product_name
        ).filter(
            models.SlcRadProduct.id == models.SlcMemberOrder.product_id,
            models.SlcMemberOrder.account_number==account_number
        ).order_by(models.SlcMemberOrder.create_time.desc())
        
    accepts = db.query(
        models.SlcRadAcceptLog.id,
        models.SlcRadAcceptLog.accept_type,
        models.SlcRadAcceptLog.accept_time,
        models.SlcRadAcceptLog.accept_desc,
        models.SlcRadAcceptLog.operator_name,
        models.SlcRadAcceptLog.accept_source,
        models.SlcRadAcceptLog.account_number,
        models.SlcMember.node_id,
        models.SlcNode.node_name
    ).filter(
            models.SlcRadAcceptLog.account_number == models.SlcRadAccount.account_number,
            models.SlcMember.member_id == models.SlcRadAccount.member_id,
            models.SlcNode.id == models.SlcMember.node_id,
            models.SlcRadAcceptLog.account_number == account_number
    ).order_by(models.SlcRadAcceptLog.accept_time.desc())
    get_orderid = lambda aid:db.query(models.SlcMemberOrder.order_id).filter_by(accept_id=aid).scalar()
    type_map = ACCEPT_TYPES
    return  render("bus_member_detail",
        member=member,
        user=user,
        orders=orders,
        accepts=accepts,
        type_map=type_map,
        get_orderid=get_orderid
    )

permit.add_route("/bus/member/detail",u"用户详情查看",u"营业管理",order=0.02)

member_detail_url_formatter = "/bus/member/detail?account_number={0}".format

###############################################################################
# member delete
###############################################################################
@app.get('/member/delete',apply=auth_opr)
def member_delete(db):
    member_id = request.params.get("member_id")
    if not member_id:
        raise abort(404,'member_id is empty')
    db.query(models.SlcMember).filter_by(member_id=member_id).delete()
    for account in db.query(models.SlcRadAccount).filter_by(member_id=member_id):
        db.query(models.SlcRadAcceptLog).filter_by(account_number=account.account_number).delete()
        db.query(models.SlcRadAccountAttr).filter_by(account_number=account.account_number).delete()
        db.query(models.SlcRadBilling).filter_by(account_number=account.account_number).delete()
        db.query(models.SlcRadTicket).filter_by(account_number=account.account_number).delete()
        db.query(models.SlcRadOnline).filter_by(account_number=account.account_number).delete()
        db.query(models.SlcRechargeLog).filter_by(account_number=account.account_number).delete()
    db.query(models.SlcRadAccount).filter_by(member_id=member_id).delete()
    db.query(models.SlcMemberOrder).filter_by(member_id=member_id).delete()

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)删除用户%s'%(get_cookie("username"),member_id)
    db.add(ops_log)
    
    db.commit()
    
    return redirect("/bus/member")
    
permit.add_route("/bus/member/delete",u"删除用户信息",u"营业管理",order=0.03)

###############################################################################
# member update
###############################################################################

@app.get('/member/update',apply=auth_opr)
def member_update(db):
    member_id = request.params.get("member_id")
    account_number = request.params.get("account_number")
    member = db.query(models.SlcMember).get(member_id)
    nodes = [ (n.id,n.node_name) for n in get_opr_nodes(db)]
    form = forms.member_update_form(nodes)
    form.fill(member)
    form.account_number.set_value(account_number)
    return render("base_form",form=form)

@app.post('/member/update',apply=auth_opr)
def member_update(db):
    nodes = [ (n.id,n.node_name) for n in get_opr_nodes(db)]
    form=forms.member_update_form(nodes)
    if not form.validates(source=request.forms):
        return render("base_form", form=form)

    member = db.query(models.SlcMember).get(form.d.member_id)
    member.realname = form.d.realname
    if form.d.new_password:
        member.password =  md5(form.d.new_password.encode()).hexdigest()
    member.email = form.d.email
    member.idcard = form.d.idcard
    member.mobile = form.d.mobile
    member.address = form.d.address

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)修改用户信息:%s'%(get_cookie("username"),member.member_name)
    db.add(ops_log)

    db.commit()
    redirect(member_detail_url_formatter(form.d.account_number))

permit.add_route("/bus/member/update",u"修改用户资料",u"营业管理",order=0.04)

###############################################################################
# member open
###############################################################################

@app.get('/member/open',apply=auth_opr)
def member_open(db):
    nodes = [ (n.id,n.node_desc) for n in get_opr_nodes(db)]
    products = [ (n.id,n.product_name) for n in db.query(models.SlcRadProduct).filter_by(
        product_status = 0
    )]
    form = forms.user_open_form(nodes,products)
    return render("bus_open_form",form=form)

@app.post('/member/open',apply=auth_opr)
def member_open(db):
    nodes = [ (n.id,n.node_desc) for n in get_opr_nodes(db)]
    products = [ (n.id,n.product_name) for n in db.query(models.SlcRadProduct).filter_by(
        product_status = 0
    )]
    form = forms.user_open_form(nodes,products)
    if not form.validates(source=request.forms):
        return render("bus_open_form", form=form)

    if db.query(models.SlcRadAccount).filter_by(account_number=form.d.account_number).count()>0:
        return render("bus_open_form", form=form,msg=u"上网账号%s已经存在"%form.d.account_number)

    if form.d.ip_address and db.query(models.SlcRadAccount).filter_by(ip_address=form.d.ip_address).count()>0:
        return render("bus_open_form", form=form,msg=u"ip%s已经被使用"%form.d.ip_address)

    if db.query(models.SlcMember).filter_by(
        member_name=form.d.member_name).count()>0:
        return render("bus_open_form", form=form,msg=u"用户名%s已经存在"%form.d.member_name)

    member = models.SlcMember()
    member.node_id = form.d.node_id
    member.realname = form.d.realname
    member.member_name = form.d.member_name or form.d.account_number
    mpwd = form.d.member_password or form.d.password
    member.password = md5(mpwd.encode()).hexdigest()
    member.idcard = form.d.idcard
    member.sex = '1'
    member.age = '0'
    member.email = ''
    member.mobile = form.d.mobile
    member.address = form.d.address
    member.create_time = utils.get_currtime()
    member.update_time = utils.get_currtime()
    member.email_active = 0
    member.mobile_active = 0
    member.active_code = utils.get_uuid()
    db.add(member)
    db.flush()
    db.refresh(member)

    accept_log = models.SlcRadAcceptLog()
    accept_log.accept_type = 'open'
    accept_log.accept_source = 'console'
    accept_log.account_number = form.d.account_number
    accept_log.accept_time = member.create_time
    accept_log.operator_name = get_cookie("username")
    accept_log.accept_desc = u"用户新开户：(%s)%s"%(member.member_name,member.realname)
    db.add(accept_log)
    db.flush()
    db.refresh(accept_log)

    order_fee = 0
    balance = 0
    expire_date = form.d.expire_date
    product = db.query(models.SlcRadProduct).get(form.d.product_id)
    # 预付费包月
    if product.product_policy == 0:
        order_fee = decimal.Decimal(product.fee_price) * decimal.Decimal(form.d.months)
        order_fee = int(order_fee.to_integral_value())
    # 买断包月,买断流量
    elif product.product_policy in (2,5):
        order_fee = int(product.fee_price)
    #预付费时长,预付费流量
    elif product.product_policy in (1,4):
        balance = utils.yuan2fen(form.d.fee_value)
        expire_date = '3000-11-11'

    order = models.SlcMemberOrder()
    order.order_id = utils.gen_order_id()
    order.member_id = member.member_id
    order.product_id = product.id
    order.account_number = form.d.account_number
    order.order_fee = order_fee
    order.actual_fee = utils.yuan2fen(form.d.fee_value)
    order.pay_status = 1
    order.accept_id = accept_log.id
    order.order_source = 'console'
    order.create_time = member.create_time
    order.order_desc = u"用户新开账号"
    db.add(order)

    account = models.SlcRadAccount()
    account.account_number = form.d.account_number
    account.ip_address = form.d.ip_address
    account.member_id = member.member_id
    account.product_id = order.product_id
    account.install_address = member.address
    account.mac_addr = ''
    account.password = utils.encrypt(form.d.password)
    account.status = form.d.status
    account.balance = balance
    account.time_length = int(product.fee_times)
    account.flow_length = int(product.fee_flows)
    account.expire_date = expire_date
    account.user_concur_number = product.concur_number
    account.bind_mac = product.bind_mac
    account.bind_vlan = product.bind_vlan
    account.vlan_id = 0
    account.vlan_id2 = 0
    account.create_time = member.create_time
    account.update_time = member.create_time
    db.add(account)

    db.commit()
    redirect(member_detail_url_formatter(account.account_number))

permit.add_route("/bus/member/open",u"用户快速开户",u"营业管理",is_menu=True,order=1)

###############################################################################
# account open
###############################################################################

@app.get('/account/open',apply=auth_opr)
def account_open(db):
    member_id =   request.params.get('member_id')
    member = db.query(models.SlcMember).get(member_id)
    products = [ (n.id,n.product_name) for n in db.query(models.SlcRadProduct).filter_by(
        product_status = 0
    )]
    form = forms.account_open_form(products)
    form.member_id.set_value(member_id)
    form.realname.set_value(member.realname)
    form.node_id.set_value(member.node_id)
    return render("bus_open_form",form=form)

@app.post('/account/open',apply=auth_opr)
def account_open(db):
    products = [ (n.id,n.product_name) for n in db.query(models.SlcRadProduct).filter_by(
        product_status = 0
    )]
    form = forms.account_open_form(products)
    if not form.validates(source=request.forms):
        return render("bus_open_form", form=form)

    if db.query(models.SlcRadAccount).filter_by(
        account_number=form.d.account_number).count()>0:
        return render("bus_open_form", form=form,msg=u"上网账号已经存在")

    if form.d.ip_address and db.query(models.SlcRadAccount).filter_by(ip_address=form.d.ip_address).count()>0:
        return render("bus_open_form", form=form,msg=u"ip%s已经被使用"%form.d.ip_address)

    accept_log = models.SlcRadAcceptLog()
    accept_log.accept_type = 'open'
    accept_log.accept_source = 'console'
    accept_log.account_number = form.d.account_number
    accept_log.accept_time = utils.get_currtime()
    accept_log.operator_name = get_cookie("username")
    accept_log.accept_desc = u"用户新增账号：上网账号:%s"%(form.d.account_number)
    db.add(accept_log)
    db.flush()
    db.refresh(accept_log)

    _datetime = utils.get_currtime()
    order_fee = 0
    balance = 0
    expire_date = form.d.expire_date
    product = db.query(models.SlcRadProduct).get(form.d.product_id)
    # 预付费包月
    if product.product_policy == 0:
        order_fee = decimal.Decimal(product.fee_price) * decimal.Decimal(form.d.months)
        order_fee = int(order_fee.to_integral_value())
    # 买断包月,买断时长,买断流量
    elif product.product_policy in (2,3,5):
        order_fee = int(product.fee_price)
    #预付费时长,预付费流量
    elif product.product_policy in (1,4):
        balance = utils.yuan2fen(form.d.fee_value)
        expire_date = '3000-11-11'

    order = models.SlcMemberOrder()
    order.order_id = utils.gen_order_id()
    order.member_id = form.d.member_id
    order.product_id = product.id
    order.account_number = form.d.account_number
    order.order_fee = order_fee
    order.actual_fee = utils.yuan2fen(form.d.fee_value)
    order.pay_status = 1
    order.accept_id = accept_log.id
    order.order_source = 'console'
    order.create_time = _datetime
    order.order_desc = u"用户增开账号"
    db.add(order)

    account = models.SlcRadAccount()
    account.account_number = form.d.account_number
    account.ip_address = form.d.ip_address
    account.member_id = int(form.d.member_id)
    account.product_id = order.product_id
    account.install_address = form.d.address
    account.mac_addr = ''
    account.password = utils.encrypt(form.d.password)
    account.status = form.d.status
    account.balance = balance
    account.time_length = int(product.fee_times)
    account.flow_length = int(product.fee_flows)
    account.expire_date = expire_date
    account.user_concur_number = product.concur_number
    account.bind_mac = product.bind_mac
    account.bind_vlan = product.bind_vlan
    account.vlan_id = 0
    account.vlan_id2 = 0
    account.create_time = _datetime
    account.update_time = _datetime
    db.add(account)

    db.commit()
    redirect(member_detail_url_formatter(account.account_number))

permit.add_route("/bus/account/open",u"增开用户账号",u"营业管理",order=1.01)

###############################################################################
# account update
###############################################################################

@app.get('/account/update',apply=auth_opr)
def account_update(db):
    account_number = request.params.get("account_number")
    account = db.query(models.SlcRadAccount).get(account_number)
    form = forms.account_update_form()
    form.fill(account)
    return render("base_form",form=form)

@app.post('/account/update',apply=auth_opr)
def account_update(db):
    form = forms.account_update_form()
    if not form.validates(source=request.forms):
        return render("base_form", form=form)

    account = db.query(models.SlcRadAccount).get(form.d.account_number)
    account.ip_address = form.d.ip_address
    account.install_address = form.d.install_address
    account.user_concur_number = form.d.user_concur_number
    account.bind_mac = form.d.bind_mac
    account.bind_vlan = form.d.bind_vlan
    if form.d.new_password:
        account.password =  utils.encrypt(form.d.new_password)

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)修改上网账号信息:%s'%(get_cookie("username"),account.account_number)
    db.add(ops_log)

    db.commit()
    websock.update_cache("account",account_number=account.account_number)
    redirect(member_detail_url_formatter(account.account_number))

permit.add_route("/bus/account/update",u"修改用户上网账号",u"营业管理",order=1.02)

###############################################################################
# account import
###############################################################################

@app.get('/member/import',apply=auth_opr)
def member_import(db):
    nodes = [ (n.id,n.node_desc) for n in get_opr_nodes(db)]
    products = [(p.id,p.product_name) for p in db.query(models.SlcRadProduct)]
    form = forms.user_import_form(nodes,products)
    return render("bus_import_form",form=form)

@app.post('/member/import',apply=auth_opr)
def member_import(db):
    nodes = [ (n.id,n.node_desc) for n in get_opr_nodes(db)]
    products = [(p.id,p.product_name) for p in db.query(models.SlcRadProduct)]
    iform = forms.user_import_form(nodes,products)
    node_id =   request.params.get('node_id')
    product_id =   request.params.get('product_id')
    upload = request.files.get('import_file')
    impctx = upload.file.read()
    lines = impctx.split("\n")
    _num = 0
    impusers = []
    for line in lines:
        _num += 1
        line = line.strip()
        if not line or "用户姓名" in line:continue
        attr_array = line.split(",")
        if len(attr_array) < 7:
            return render("bus_import_form",form=iform,msg=u"line %s error: length must 7 "%_num)

        vform = forms.user_import_vform()
        if not vform.validates(dict(
                realname = attr_array[0],
                idcard = attr_array[1],
                mobile = attr_array[2],
                address = attr_array[3],
                account_number = attr_array[4],
                password = attr_array[5],
                begin_date = attr_array[6],
                expire_date = attr_array[7],
                balance = attr_array[8],
                time_length = utils.hour2sec(attr_array[9]),
                flow_length = utils.mb2kb(attr_array[10]))):
            return render("bus_import_form",form=iform,msg=u"line %s error: %s"%(_num,vform.errors))

        impusers.append(vform)

    for form in impusers:
        try:
            member = models.SlcMember()
            member.node_id = node_id
            member.realname = form.d.realname
            member.idcard = form.d.idcard
            member.member_name = form.d.account_number
            member.password = md5(form.d.password.encode()).hexdigest()
            member.sex = '1'
            member.age = '0'
            member.email = ''
            member.mobile = form.d.mobile
            member.address = form.d.address
            member.create_time = form.d.begin_date + ' 00:00:00'
            member.update_time = utils.get_currtime()
            member.email_active = 0
            member.mobile_active = 0
            member.active_code = utils.get_uuid()
            db.add(member)
            db.flush()
            db.refresh(member)

            accept_log = models.SlcRadAcceptLog()
            accept_log.accept_type = 'open'
            accept_log.accept_source = 'console'
            _desc = u"用户导入账号：%s"% form.d.account_number
            accept_log.accept_desc = _desc
            accept_log.account_number = form.d.account_number
            accept_log.accept_time = member.update_time
            accept_log.operator_name = get_cookie("username")
            db.add(accept_log)
            db.flush()
            db.refresh(accept_log)

            order_fee = 0
            actual_fee = 0
            balance = 0
            time_length = 0
            flow_length = 0
            expire_date = form.d.expire_date
            product = db.query(models.SlcRadProduct).get(product_id)
            # 买断时长
            if product.product_policy == 3:
                time_length = int(form.d.time_length)
            # 买断流量
            elif product.product_policy == 5:
                flow_length = int(form.d.flow_length)
            #预付费时长,预付费流量
            elif product.product_policy in (1,4):
                balance = utils.yuan2fen(form.d.balance)
                expire_date = '3000-11-11'                

            order = models.SlcMemberOrder()
            order.order_id = utils.gen_order_id()
            order.member_id = member.member_id
            order.product_id = product.id
            order.account_number = form.d.account_number
            order.order_fee = order_fee
            order.actual_fee = actual_fee
            order.pay_status = 1
            order.accept_id = accept_log.id
            order.order_source = 'console'
            order.create_time = member.update_time
            order.order_desc = u"用户导入开户"
            db.add(order)

            account = models.SlcRadAccount()
            account.account_number = form.d.account_number
            account.member_id = member.member_id
            account.product_id = order.product_id
            account.install_address = member.address
            account.ip_address = ''
            account.mac_addr = ''
            account.password = utils.encrypt(form.d.password)
            account.status = 1
            account.balance = balance
            account.time_length = time_length
            account.flow_length = flow_length
            account.expire_date = expire_date
            account.user_concur_number = product.concur_number
            account.bind_mac = product.bind_mac
            account.bind_vlan = product.bind_vlan
            account.vlan_id = 0
            account.vlan_id2 = 0
            account.create_time = member.create_time
            account.update_time = member.update_time
            db.add(account)

        except Exception as e:
            return render("bus_import_form",form=iform,msg=u"error : %s"%str(e))

    db.commit()
    redirect("/bus/member")

permit.add_route("/bus/member/import",u"用户数据导入",u"营业管理",is_menu=True,order=2)

###############################################################################
# account pause
###############################################################################

@app.post('/account/pause',apply=auth_opr)
def account_pause(db):
    account_number = request.params.get("account_number")
    account = db.query(models.SlcRadAccount).get(account_number)

    if account.status != 1:
        return dict(msg=u"用户当前状态不允许停机")

    _datetime = utils.get_currtime()
    account.last_pause = _datetime
    account.status = 2

    accept_log = models.SlcRadAcceptLog()
    accept_log.accept_type = 'pause'
    accept_log.accept_source = 'console'
    accept_log.accept_desc = u"用户停机：上网账号:%s"%(account_number)
    accept_log.account_number = account.account_number
    accept_log.accept_time = _datetime
    accept_log.operator_name = get_cookie("username")
    db.add(accept_log)

    db.commit()
    websock.update_cache("account",account_number=account.account_number)

    onlines = db.query(models.SlcRadOnline).filter_by(account_number=account_number)
    for _online in onlines:
        websock.invoke_admin("coa_request",
            nas_addr=_online.nas_addr,
            acct_session_id=_online.acct_session_id,
            message_type='disconnect')
    return dict(msg=u"操作成功")

permit.add_route("/bus/account/pause",u"用户账号停机",u"营业管理",order=2.01)

###############################################################################
# account resume
###############################################################################

@app.post('/account/resume',apply=auth_opr)
def account_resume(db):
    account_number = request.params.get("account_number")
    account = db.query(models.SlcRadAccount).get(account_number)
    if account.status != 2:
        return dict(msg=u"用户当前状态不允许复机")

    account.status = 1
    _datetime = datetime.datetime.now()
    _pause_time = datetime.datetime.strptime(account.last_pause,"%Y-%m-%d %H:%M:%S")
    _expire_date = datetime.datetime.strptime(account.expire_date+' 23:59:59',"%Y-%m-%d %H:%M:%S")
    days = (_expire_date - _pause_time).days
    new_expire = (_datetime + datetime.timedelta(days=int(days))).strftime("%Y-%m-%d")
    account.expire_date = new_expire

    accept_log = models.SlcRadAcceptLog()
    accept_log.accept_type = 'resume'
    accept_log.accept_source = 'console'
    accept_log.accept_desc = u"用户复机：上网账号:%s"%(account_number)
    accept_log.account_number = account.account_number
    accept_log.accept_time = utils.get_currtime()
    accept_log.operator_name = get_cookie("username")
    db.add(accept_log)

    db.commit()
    websock.update_cache("account",account_number=account.account_number)
    return dict(msg=u"操作成功")

permit.add_route("/bus/account/resume",u"用户账号复机",u"营业管理",order=2.02)


def query_account(db,account_number):
    return db.query(
        models.SlcMember.realname,
        models.SlcRadAccount.member_id,
        models.SlcRadAccount.product_id,
        models.SlcRadAccount.account_number,
        models.SlcRadAccount.expire_date,
        models.SlcRadAccount.balance,
        models.SlcRadAccount.time_length,
        models.SlcRadAccount.flow_length,
        models.SlcRadAccount.user_concur_number,
        models.SlcRadAccount.status,
        models.SlcRadAccount.mac_addr,
        models.SlcRadAccount.vlan_id,
        models.SlcRadAccount.vlan_id2,
        models.SlcRadAccount.ip_address,
        models.SlcRadAccount.bind_mac,
        models.SlcRadAccount.bind_vlan,
        models.SlcRadAccount.ip_address,
        models.SlcRadAccount.install_address,
        models.SlcRadAccount.create_time,
        models.SlcRadProduct.product_name
    ).filter(
            models.SlcRadProduct.id == models.SlcRadAccount.product_id,
            models.SlcMember.member_id == models.SlcRadAccount.member_id,
            models.SlcRadAccount.account_number == account_number
    ).first()


###############################################################################
# account next
###############################################################################

@app.get('/account/next',apply=auth_opr)
def account_next(db):
    account_number = request.params.get("account_number")
    user = query_account(db,account_number)
    form = forms.account_next_form()
    form.account_number.set_value(account_number)
    form.old_expire.set_value(user.expire_date)
    form.product_id.set_value(user.product_id)
    return render("bus_account_next_form",user=user,form=form)

@app.post('/account/next',apply=auth_opr)
def account_next(db):
    account_number = request.params.get("account_number")
    account = db.query(models.SlcRadAccount).get(account_number)
    user = query_account(db,account_number)
    form = forms.account_next_form()
    form.product_id.set_value(user.product_id)
    if account.status not in (1,4):
        return render("bus_account_next_form", user=user,form=form,msg=u"无效用户状态")
    if not form.validates(source=request.forms):
        return render("bus_account_next_form", user=user,form=form)

    accept_log = models.SlcRadAcceptLog()
    accept_log.accept_type = 'next'
    accept_log.accept_source = 'console'
    accept_log.accept_desc = u"用户续费：上网账号:%s，续费%s元"%(account_number,form.d.fee_value)
    accept_log.account_number = form.d.account_number
    accept_log.accept_time = utils.get_currtime()
    accept_log.operator_name = get_cookie("username")
    db.add(accept_log)
    db.flush()
    db.refresh(accept_log)

    order_fee = 0
    product = db.query(models.SlcRadProduct).get(user.product_id)
    
    # 预付费包月
    if product.product_policy == PPMonth:
        order_fee = decimal.Decimal(product.fee_price) * decimal.Decimal(form.d.months)
        order_fee = int(order_fee.to_integral_value())
    # 买断包月,买断流量,买断时长
    elif product.product_policy in (BOMonth,BOTimes,BOFlows):
        order_fee = int(product.fee_price)


    order = models.SlcMemberOrder()
    order.order_id = utils.gen_order_id()
    order.member_id = user.member_id
    order.product_id = user.product_id
    order.account_number = form.d.account_number
    order.order_fee = order_fee
    order.actual_fee = utils.yuan2fen(form.d.fee_value)
    order.pay_status = 1
    order.accept_id = accept_log.id
    order.order_source = 'console'
    order.create_time = utils.get_currtime()
    order.order_desc = u"用户续费"
    db.add(order)

    account.status = 1
    account.expire_date = form.d.expire_date
    if product.product_policy == BOTimes:
        account.time_length += product.fee_times
    elif product.product_policy == BOFlows:
        account.flow_length += product.fee_flows

    db.commit()
    websock.update_cache("account",account_number=account_number)
    redirect(member_detail_url_formatter(account_number))

permit.add_route("/bus/account/next",u"用户账号续费",u"营业管理",order=2.03)

###############################################################################
# account charge
###############################################################################

@app.get('/account/charge',apply=auth_opr)
def account_charge(db):
    account_number = request.params.get("account_number")
    user = query_account(db,account_number)
    form = forms.account_charge_form()
    form.account_number.set_value(account_number)
    return render("bus_account_form",user=user,form=form)

@app.post('/account/charge',apply=auth_opr)
def account_charge(db):
    account_number = request.params.get("account_number")
    account = db.query(models.SlcRadAccount).get(account_number)
    user = query_account(db,account_number)
    form = forms.account_charge_form()
    if account.status !=1 :
        return render("bus_account_form", user=user,form=form,msg=u"无效用户状态")

    if not form.validates(source=request.forms):
        return render("bus_account_form", user=user,form=form)

    accept_log = models.SlcRadAcceptLog()
    accept_log.accept_type = 'charge'
    accept_log.accept_source = 'console'
    accept_log.account_number = form.d.account_number
    accept_log.accept_time = utils.get_currtime()
    accept_log.operator_name = get_cookie("username")
    _new_fee = account.balance + utils.yuan2fen(form.d.fee_value)
    accept_log.accept_desc = u"用户充值：充值前%s元,充值后%s元"%(
        utils.fen2yuan(account.balance),
        utils.fen2yuan(_new_fee)
    )
    db.add(accept_log)
    db.flush()
    db.refresh(accept_log)

    order = models.SlcMemberOrder()
    order.order_id = utils.gen_order_id()
    order.member_id = user.member_id
    order.product_id = user.product_id
    order.account_number = form.d.account_number
    order.order_fee = 0
    order.actual_fee = utils.yuan2fen(form.d.fee_value)
    order.pay_status = 1
    order.accept_id = accept_log.id
    order.order_source = 'console'
    order.create_time = utils.get_currtime()
    order.order_desc = accept_log.accept_desc
    db.add(order)

    account.balance += order.actual_fee

    db.commit()
    websock.update_cache("account",account_number=account_number)
    redirect(member_detail_url_formatter(account_number))

permit.add_route("/bus/account/charge",u"用户账号充值",u"营业管理",order=2.04)

###############################################################################
# account product change
###############################################################################

@app.get('/account/change',apply=auth_opr)
def account_change(db):
    account_number = request.params.get("account_number")
    products = [(p.id,p.product_name) for p in db.query(models.SlcRadProduct)]
    user = query_account(db,account_number)
    form = forms.account_change_form(products=products)
    form.account_number.set_value(account_number)
    return render("bus_account_change_form",user=user,form=form)
    
@app.post('/account/change',apply=auth_opr)
def account_change(db):
    account_number = request.params.get("account_number")
    products = [(p.id,p.product_name) for p in db.query(models.SlcRadProduct)]
    form = forms.account_change_form(products=products)
    account = db.query(models.SlcRadAccount).get(account_number)
    user = query_account(db,account_number)
    if account.status not in (1,4):
        return render("bus_account_change_form", user=user,form=form,msg=u"无效用户状态")
    if not form.validates(source=request.forms):
        return render("bus_account_change_form", user=user,form=form)
        
    product = db.query(models.SlcRadProduct).get(form.d.product_id)

    accept_log = models.SlcRadAcceptLog()
    accept_log.accept_type = 'change'
    accept_log.accept_source = 'console'
    accept_log.account_number = form.d.account_number
    accept_log.accept_time = utils.get_currtime()
    accept_log.operator_name = get_cookie("username")
    accept_log.accept_desc = u"用户资费变更为:%s"%(product.product_name)
    db.add(accept_log)
    db.flush()
    db.refresh(accept_log)
    
    account.product_id = product.id
    #(PPMonth,PPTimes,BOMonth,BOTimes,PPFlow,BOFlows)
    if product.product_policy in (PPMonth,BOMonth):
        account.expire_date = form.d.expire_date
        account.balance = 0
        account.time_length = 0
        account.flow_length = 0
    elif product.product_policy in (PPTimes,PPFlow):
        account.expire_date = MAX_EXPIRE_DATE
        account.balance = utils.yuan2fen(form.d.balance)
        account.time_length = 0
        account.flow_length = 0
    elif product.product_policy == BOTimes:
        account.expire_date = MAX_EXPIRE_DATE
        account.balance = 0
        account.time_length = utils.hour2sec(form.d.time_length)
        account.flow_length = 0
    elif product.product_policy == BOFlows:
        account.expire_date = MAX_EXPIRE_DATE
        account.balance = 0
        account.time_length = 0
        account.flow_length = utils.mb2kb(form.d.flow_length)

    order = models.SlcMemberOrder()
    order.order_id = utils.gen_order_id()
    order.member_id = account.member_id
    order.product_id = account.product_id
    order.account_number = account.account_number
    order.order_fee = 0
    order.actual_fee = utils.yuan2fen(form.d.add_value) - utils.yuan2fen(form.d.back_value)
    order.pay_status = 1
    order.accept_id = accept_log.id
    order.order_source = 'console'
    order.create_time = utils.get_currtime()
    order.order_desc =  u"用户资费变更，费用:%s元"%utils.fen2yuan(order.actual_fee)
    db.add(order)
    

    db.commit()
    websock.update_cache("account",account_number=account_number)
    redirect(member_detail_url_formatter(account_number))
    
permit.add_route("/bus/account/change",u"用户资费变更",u"营业管理",order=2.05)

###############################################################################
# account cancel
###############################################################################

@app.get('/account/cancel',apply=auth_opr)
def account_cancel(db):
    account_number = request.params.get("account_number")
    user = query_account(db,account_number)
    form = forms.account_cancel_form()
    form.account_number.set_value(account_number)
    return render("bus_account_form",user=user,form=form)


@app.post('/account/cancel',apply=auth_opr)
def account_cancel(db):
    account_number = request.params.get("account_number")
    account = db.query(models.SlcRadAccount).get(account_number)
    user = query_account(db,account_number)
    form = forms.account_cancel_form()
    if account.status !=1 :
        return render("bus_account_form", user=user,form=form,msg=u"无效用户状态")
    if not form.validates(source=request.forms):
        return render("bus_account_form", user=user,form=form)

    accept_log = models.SlcRadAcceptLog()
    accept_log.accept_type = 'cancel'
    accept_log.accept_source = 'console'
    accept_log.account_number = form.d.account_number
    accept_log.accept_time = utils.get_currtime()
    accept_log.operator_name = get_cookie("username")
    accept_log.accept_desc = u"用户销户退费%s(元)"%(form.d.fee_value)
    db.add(accept_log)
    db.flush()
    db.refresh(accept_log)

    order = models.SlcMemberOrder()
    order.order_id = utils.gen_order_id()
    order.member_id = user.member_id
    order.product_id = user.product_id
    order.account_number = form.d.account_number
    order.order_fee = 0
    order.actual_fee = -utils.yuan2fen(form.d.fee_value)
    order.pay_status = 1
    order.order_source = 'console'
    order.accept_id = accept_log.id
    order.create_time = utils.get_currtime()
    order.order_desc = accept_log.accept_desc
    db.add(order)

    account.status = 3

    db.commit()

    websock.update_cache("account",account_number=account_number)
    onlines = db.query(models.SlcRadOnline).filter_by(account_number=account_number)
    for _online in onlines:
        websock.invoke_admin("coa_request",
            nas_addr=_online.nas_addr,
            acct_session_id=_online.acct_session_id,
            message_type='disconnect')
    redirect(member_detail_url_formatter(account_number))

permit.add_route("/bus/account/cancel",u"用户账号销户",u"营业管理",order=2.05)

###############################################################################
# accept log manage
###############################################################################

@app.route('/acceptlog',apply=auth_opr,method=['GET','POST'])
@app.post('/acceptlog/export',apply=auth_opr)
def acceptlog_query(db):
    node_id = request.params.get('node_id')
    accept_type = request.params.get('accept_type')
    account_number = request.params.get('account_number')
    operator_name = request.params.get('operator_name')
    query_begin_time = request.params.get('query_begin_time')
    query_end_time = request.params.get('query_end_time')
    opr_nodes = get_opr_nodes(db)
    _query = db.query(
        models.SlcRadAcceptLog.id,
        models.SlcRadAcceptLog.accept_type,
        models.SlcRadAcceptLog.accept_time,
        models.SlcRadAcceptLog.accept_desc,
        models.SlcRadAcceptLog.operator_name,
        models.SlcRadAcceptLog.accept_source,
        models.SlcRadAcceptLog.account_number,
        models.SlcMember.node_id,
        models.SlcNode.node_name
    ).filter(
            models.SlcRadAcceptLog.account_number == models.SlcRadAccount.account_number,
            models.SlcMember.member_id == models.SlcRadAccount.member_id,
            models.SlcNode.id == models.SlcMember.node_id
    )
    if operator_name:
        _query = _query.filter(models.SlcRadAcceptLog.operator_name == operator_name)
    if node_id:
        _query = _query.filter(models.SlcMember.node_id == node_id)
    else:
        _query = _query.filter(models.SlcMember.node_id.in_([i.id for i in opr_nodes]))
    if account_number:
        _query = _query.filter(models.SlcRadAcceptLog.account_number.like('%'+account_number+'%'))
    if accept_type:
        _query = _query.filter(models.SlcRadAcceptLog.accept_type == accept_type)
    if query_begin_time:
        _query = _query.filter(models.SlcRadAcceptLog.accept_time >= query_begin_time+' 00:00:00')
    if query_end_time:
        _query = _query.filter(models.SlcRadAcceptLog.accept_time <= query_end_time+' 23:59:59')
    _query = _query.order_by(models.SlcRadAcceptLog.accept_time.desc())
    type_map = ACCEPT_TYPES
    if request.path == '/acceptlog':
        return render(
            "bus_acceptlog_list",
            page_data = get_page_data(_query),
            node_list=opr_nodes,
            type_map = type_map,
            get_orderid = lambda aid:db.query(models.SlcMemberOrder.order_id).filter_by(accept_id=aid).scalar(),
            **request.params
        )
    elif request.path == '/acceptlog/export':
        data = Dataset()
        data.append((u'区域',u'上网账号',u'受理类型',u'受理时间',u'受理渠道',u'操作员',u'受理描述'))
        for i in _query:
            data.append((
                i.node_name, i.account_number, type_map.get(i.accept_type),
                i.accept_time,i.accept_source,i.operator_name,i.accept_desc
            ))
        name = u"RADIUS-ACCEPTLOG-" + datetime.datetime.now().strftime("%Y%m%d-%H%M%S") + ".xls"
        return export_file(name,data)

permit.add_route("/bus/acceptlog",u"用户受理查询",u"营业管理",is_menu=True,order=3)
permit.add_route("/bus/acceptlog/export",u"用户受理导出",u"营业管理",order=3.01)

###############################################################################
# member update
###############################################################################
@app.get('/account/delete',apply=auth_opr)
def account_delete(db):
    account_number = request.params.get("account_number")
    if not account_number:
        raise abort(404,'account_number is empty')
    account = db.query(models.SlcRadAccount).get(account_number)
    member_id = account.member_id
        
    db.query(models.SlcRadAcceptLog).filter_by(account_number=account.account_number).delete()
    db.query(models.SlcRadAccountAttr).filter_by(account_number=account.account_number).delete()
    db.query(models.SlcRadBilling).filter_by(account_number=account.account_number).delete()
    db.query(models.SlcRadTicket).filter_by(account_number=account.account_number).delete()
    db.query(models.SlcRadOnline).filter_by(account_number=account.account_number).delete()
    db.query(models.SlcRechargeLog).filter_by(account_number=account.account_number).delete()
    db.query(models.SlcRadAccount).filter_by(account_number=account.account_number).delete()
    db.query(models.SlcMemberOrder).filter_by(account_number=account.account_number).delete()
    
    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)删除用户账号%s'%(get_cookie("username"),account_number)
    db.add(ops_log)
    
    db.commit()
    return redirect("/bus/member")
    
permit.add_route("/bus/account/delete",u"删除用户账号",u"营业管理",order=3.02)

###############################################################################
# billing log query
###############################################################################

@app.route('/billing',apply=auth_opr,method=['GET','POST'])
@app.post('/billing/export',apply=auth_opr)
def billing_query(db):
    node_id = request.params.get('node_id')
    account_number = request.params.get('account_number')
    query_begin_time = request.params.get('query_begin_time')
    query_end_time = request.params.get('query_end_time')
    opr_nodes = get_opr_nodes(db)
    _query = db.query(
        models.SlcRadBilling,
        models.SlcMember.node_id,
        models.SlcNode.node_name
    ).filter(
        models.SlcRadBilling.account_number == models.SlcRadAccount.account_number,
        models.SlcMember.member_id == models.SlcRadAccount.member_id,
        models.SlcNode.id == models.SlcMember.node_id
    )
    if node_id:
        _query = _query.filter(models.SlcMember.node_id == node_id)
    else:
        _query = _query.filter(models.SlcMember.node_id.in_(i.id for i in opr_nodes))
    if account_number:
        _query = _query.filter(models.SlcRadBilling.account_number.like('%'+account_number+'%'))
    if query_begin_time:
        _query = _query.filter(models.SlcRadBilling.create_time >= query_begin_time+' 00:00:00')
    if query_end_time:
        _query = _query.filter(models.SlcRadBilling.create_time <= query_end_time+' 23:59:59')
    _query = _query.order_by(models.SlcRadBilling.create_time.desc())
    if request.path == '/billing':
        return render("bus_billing_list",
            node_list=opr_nodes,
            page_data=get_page_data(_query),**request.params)
    elif request.path == '/billing/export':
        data = Dataset()
        data.append((
            u'区域',u'上网账号',u'BAS地址',u'会话编号',u'记账开始时间',u'会话时长',
            u'已扣时长',u"已扣流量",u'应扣费用',u'实扣费用',u'剩余余额',
            u'剩余时长',u'剩余流量',u'是否扣费',u'扣费时间'
        ))
        _f2y = utils.fen2yuan
        _fms = utils.fmt_second
        _k2m = utils.kb2mb
        _s2h = utils.sec2hour
        for i,_,_node_name in _query:
            data.append((
                _node_name, i.account_number, i.nas_addr,i.acct_session_id,
                i.acct_start_time,_fms(i.acct_session_time),_fms(i.acct_times),_k2m(i.acct_flows),
                _f2y(i.acct_fee),_f2y(i.actual_fee),_f2y(i.balance),
                _s2h(i.time_length),_k2m(i.flow_length),
                (i.is_deduct==0 and u'否' or u'是'),i.create_time
            ))
        name = u"RADIUS-BILLING-" + datetime.datetime.now().strftime("%Y%m%d-%H%M%S") + ".xls"
        return export_file(name,data)

permit.add_route("/bus/billing",u"用户计费查询",u"营业管理",is_menu=True,order=4)
permit.add_route("/bus/billing/export",u"用户计费导出",u"营业管理",order=4.01)

###############################################################################
# billing log query
###############################################################################

@app.route('/orders',apply=auth_opr,method=['GET','POST'])
@app.post('/orders/export',apply=auth_opr)
def order_query(db):
    node_id = request.params.get('node_id')
    product_id = request.params.get('product_id')
    pay_status = request.params.get('pay_status')
    account_number = request.params.get('account_number')
    query_begin_time = request.params.get('query_begin_time')
    query_end_time = request.params.get('query_end_time')
    opr_nodes = get_opr_nodes(db)
    _query = db.query(
        models.SlcMemberOrder,
        models.SlcMember.node_id,
        models.SlcMember.realname,
        models.SlcRadProduct.product_name,
        models.SlcNode.node_name
    ).filter(
        models.SlcMemberOrder.product_id == models.SlcRadProduct.id,
        models.SlcMemberOrder.member_id == models.SlcMember.member_id,
        models.SlcNode.id == models.SlcMember.node_id
    )
    if node_id:
        _query = _query.filter(models.SlcMember.node_id == node_id)
    else:
        _query = _query.filter(models.SlcMember.node_id.in_([i.id for i in opr_nodes]))
    if account_number:
        _query = _query.filter(models.SlcMemberOrder.account_number.like('%'+account_number+'%'))
    if product_id:
        _query = _query.filter(models.SlcMemberOrder.product_id == product_id)
    if pay_status:
        _query = _query.filter(models.SlcMemberOrder.pay_status == pay_status)
    if query_begin_time:
        _query = _query.filter(models.SlcMemberOrder.create_time >= query_begin_time+' 00:00:00')
    if query_end_time:
        _query = _query.filter(models.SlcMemberOrder.create_time <= query_end_time+' 23:59:59')
    _query = _query.order_by(models.SlcMemberOrder.create_time.desc())

    if request.path == '/orders':
        return render("bus_order_list",
            node_list=opr_nodes,
            products =  db.query(models.SlcRadProduct).filter_by(product_status = 0),
            page_data=get_page_data(_query),**request.params)
    elif request.path == '/orders/export':
        data = Dataset()
        data.append((
            u'区域',u"用户姓名",u'上网账号',u'资费',u"订购时间",
            u'订单费用',u'实缴费用',u'支付状态',u'订购渠道',u'订单描述'
        ))
        _f2y = utils.fen2yuan
        _fms = utils.fmt_second
        _pst = {0:u'未支付',1:u'已支付',2:u'已取消'}
        for i,_,_realname,_product_name,_node_name in _query:
            data.append((
                _node_name, _realname, i.account_number, _product_name,
                i.create_time, _f2y(i.order_fee), _f2y(i.actual_fee),
                _pst.get(i.pay_status), i.order_source, i.order_desc
            ))
        name = u"RADIUS-ORDERS-" + datetime.datetime.now().strftime("%Y%m%d-%H%M%S") + ".xls"
        return export_file(name,data)

permit.add_route("/bus/orders",u"用户交易查询",u"营业管理",is_menu=True,order=5)
permit.add_route("/bus/orders/export",u"用户交易导出",u"营业管理",order=5.01)



#coding:utf-8
import sys,os
import copy
import re
import itertools
import net

"""basic from web.py: makes web apps (http://webpy.org)"""

__version__ = "0.37"
__author__ = [
    "Aaron Swartz <me@aaronsw.com>",
    "Anand Chitipothu <anandology@gmail.com>"
]
__license__ = "public domain"
__contributors__ = "see https://github.com/webpy/webpy"

class Storage(dict):
    def __getattr__(self, key): 
        try:
            return self[key]
        except KeyError, k:
            raise AttributeError, k
    
    def __setattr__(self, key, value): 
        self[key] = value
    
    def __delattr__(self, key):
        try:
            del self[key]
        except KeyError, k:
            raise AttributeError, k
    
    def __repr__(self):     
        return '<Storage ' + dict.__repr__(self) + '>'

storage = Storage

def attrget(obj, attr, value=None):
    try:
        if hasattr(obj, 'has_key') and obj.has_key(attr): 
            return obj[attr]
    except TypeError:
        # Handle the case where has_key takes different number of arguments.
        # This is the case with Model objects on appengine. See #134
        pass
    if hasattr(obj, attr):
        return getattr(obj, attr)
    return value

def safeunicode(obj, encoding='utf-8'):

    t = type(obj)
    if t is unicode:
        return obj
    elif t is str:
        return obj.decode(encoding)
    elif t in [int, float, bool]:
        return unicode(obj)
    elif hasattr(obj, '__unicode__') or isinstance(obj, unicode):
        return unicode(obj)
    else:
        return str(obj).decode(encoding)    

def safestr(obj, encoding='utf-8'):

    if isinstance(obj, unicode):
        return obj.encode(encoding)
    elif isinstance(obj, str):
        return obj
    elif hasattr(obj, 'next'): # iterator
        return itertools.imap(safestr, obj)
    else:
        return str(obj)

def autoassign(self, locals):

    for (key, value) in locals.iteritems():
        if key == 'self': 
            continue
        setattr(self, key, value)        


class Form(object):

    def __init__(self, *inputs, **kw):
        self.inputs = inputs
        self.valid = True
        self.note = None
        self.validators = kw.pop('validators', [])
        self.action = kw.pop("action","")
        self.title = kw.pop("title","")
        self.method = kw.pop("method","post")
        self.onsubmit = kw.pop("onsubmit","")
        self.error = None

    def __call__(self, x=None):
        o = copy.deepcopy(self)
        if x: o.validates(x)
        return o
    
    def render(self):
        out = ''
        out += self.rendernote(self.note)
        out += '<table class="formtab table table-bordered">\n'
        out += '<thead ><tr class=active><th>%s</th><th class=rtd><a class="btn"\
               href="javascript:history.go(-1);">%s</a></th></tr></thead>\n'%(self.title, net.websafe("返回"))
        
        for i in self.inputs:
            html = safeunicode(i.pre) + i.render() + self.rendernote(i.note) + safeunicode(i.post)
            if i.is_hidden():
                out += '    <tr style="display: none;"><td></td><td>%s</td></tr>\n' % (html)
            else:
                out += '    <tr><td>%s</td><td>%s</td></tr>\n' % ( net.websafe(i.description),html)
        if self.error:
             out += '    <tr><td colspan=2>%s</td></tr>\n' % ( self.rendernote(self.error))
        out += "</table>"
        return out
        
    def render_css(self): 
        out = [] 
        out.append(self.rendernote(self.note)) 
        for i in self.inputs:
            out.append('    <div class="form-group">\n')
            if not i.is_hidden():
                out.append('        <label class="col-sm-4 control-label" id="lab_%s" for="%s">%s</label>\n' % (i.id,i.id, net.websafe(i.description)))
            out.append(i.pre)
            out.append('        <div class="col-sm-6">\n')
            out.append("        %s\n"%i.render())
            out.append('        </div>\n')
            if i.help:
                out.append('    <a id="%s_help" href="javascript:void" title="%s">\n'%(i.id,i.help))
                out.append('    <span class="input-help glyphicon glyphicon-question-sign"></span></a>\n')
            out.append(self.rendernote(i.note))
            out.append(i.post) 
            out.append('    </div>\n')
            if i.hr:
                out.append("<hr/>\n")
        return ''.join(out) 
        
    def rendernote(self, note):
        if note: return '<span class="wrong">%s</span>' % net.websafe(note)
        else: return ""
    
    def validates(self, source=None, _validate=True, **kw):
        source = source or kw
        out = True
        for i in self.inputs:
            v = attrget(source, i.name)
            if _validate:
                out = i.validate(v) and out
            else:
                i.set_value(v)
        if _validate:
            out = out and self._validate(source)
            self.valid = out
        return out

    def _validate(self, value):
        self.value = value
        for v in self.validators:
            if not v.valid(value):
                self.note = v.msg
                return False
        return True

    def fill(self, source=None, **kw):
        return self.validates(source, _validate=False, **kw)
    
    def __getitem__(self, i):
        for x in self.inputs:
            if x.name == i: return x
        raise KeyError, i

    def __getattr__(self, name):
        # don't interfere with deepcopy
        inputs = self.__dict__.get('inputs') or []
        for x in inputs:
            if x.name == name: return x
        raise AttributeError, name
    
    def get(self, i, default=None):
        try:
            return self[i]
        except KeyError:
            return default
            
    def _get_d(self): #@@ should really be form.attr, no?
        return storage([(i.name, i.get_value()) for i in self.inputs])
    d = property(_get_d)

class Input(object):
    def __init__(self, name, *validators, **attrs):
        self.name = name
        self.validators = validators
        self.attrs = attrs = AttributeList(attrs)
        
        self.description = attrs.pop('description', name)
        self.help = attrs.pop("help","")
        self.value = attrs.pop('value', None)
        self.pre = attrs.pop('pre', "")
        self.post = attrs.pop('post', "")
        self.hr = attrs.pop('hr',False)
        self.note = None
        
        self.id = attrs.setdefault('id', self.get_default_id())
        
        if 'class_' in attrs:
            attrs['class'] = attrs['class_']
            del attrs['class_']

        attrs['placeholder'] = self.description
        for vd in self.validators:
            attrs['placeholder'] += ", "+vd.msg



        
    def is_hidden(self):
        return False
        
    def get_type(self):
        raise NotImplementedError
        
    def get_default_id(self):
        return self.name

    def validate(self, value):
        self.set_value(value)

        for v in self.validators:
            if not v.valid(value):
                self.note = v.msg
                return False
        return True

    def set_value(self, value):
        self.value = value

    def get_value(self):
        return self.value

    def render(self):
        attrs = self.attrs.copy()
        attrs['type'] = self.get_type()
        if self.value is not None:
            attrs['value'] = self.value
        attrs['name'] = self.name
        return '<input %s/>' % attrs

    def rendernote(self, note):
        if note: return '<strong class="wrong">%s</strong>' % net.websafe(note)
        else: return ""
        
    def addatts(self):
        # add leading space for backward-compatibility
        return " " + str(self.attrs)

class AttributeList(dict):

    def copy(self):
        return AttributeList(self)
        
    def __str__(self):
        return " ".join(['%s="%s"' % (k, net.websafe(v)) for k, v in self.items()])
        
    def __repr__(self):
        return '<attrs: %s>' % repr(str(self))

class Textbox(Input):
     
    def get_type(self):
        return 'text'

class Password(Input):

    def get_type(self):
        return 'password'

class Textarea(Input):

    def render(self):
        attrs = self.attrs.copy()
        attrs['name'] = self.name
        value = net.websafe(self.value or '')
        return '<textarea %s>%s</textarea>' % (attrs, value)

class Dropdown(Input):

    def __init__(self, name, args, *validators, **attrs):
        self.args = args
        super(Dropdown, self).__init__(name, *validators, **attrs)

    def render(self):
        attrs = self.attrs.copy()
        attrs['name'] = self.name
        
        x = '<select %s>\n' % attrs
        
        for arg in self.args:
            x += self._render_option(arg)

        x += '        </select>\n'
        return x

    def _render_option(self, arg, indent='  '):
        if isinstance(arg, (tuple, list)):
            value, desc= arg
        else:
            value, desc = arg, arg 

        if self.value == value or (isinstance(self.value, list) and value in self.value):
            select_p = ' selected="selected"'
        else:
            select_p = ''
        return indent + '       <option%s value="%s">%s</option>\n' % (select_p, net.websafe(value), net.websafe(desc))
        

class GroupedDropdown(Dropdown):

    def __init__(self, name, args, *validators, **attrs):
        self.args = args
        super(Dropdown, self).__init__(name, *validators, **attrs)

    def render(self):
        attrs = self.attrs.copy()
        attrs['name'] = self.name
        
        x = '   <select %s>\n' % attrs
        
        for label, options in self.args:
            x += '  <optgroup label="%s">\n' % net.websafe(label)
            for arg in options:
                x += self._render_option(arg, indent = '    ')
            x +=  ' </optgroup>\n'
            
        x += '  </select>\n'
        return x

class Radio(Input):
    def __init__(self, name, args, *validators, **attrs):
        self.args = args
        super(Radio, self).__init__(name, *validators, **attrs)

    def render(self):
        x = '<span>'
        for arg in self.args:
            if isinstance(arg, (tuple, list)):
                value, desc= arg
            else:
                value, desc = arg, arg 
            attrs = self.attrs.copy()
            attrs['name'] = self.name
            attrs['type'] = 'radio'
            attrs['value'] = value
            if self.value == value:
                attrs['checked'] = 'checked'
            x += '<input %s/> %s' % (attrs, net.websafe(desc))
        x += '</span>'
        return x

class Checkbox(Input):

    def __init__(self, name, *validators, **attrs):
        self.checked = attrs.pop('checked', False)
        Input.__init__(self, name, *validators, **attrs)
        
    def get_default_id(self):
        value = safestr(self.value or "")
        return self.name + '_' + value.replace(' ', '_')

    def render(self):
        attrs = self.attrs.copy()
        attrs['type'] = 'checkbox'
        attrs['name'] = self.name
        attrs['value'] = self.value

        if self.checked:
            attrs['checked'] = 'checked'            
        return '<input %s/>' % attrs

    def set_value(self, value):
        self.checked = bool(value)

    def get_value(self):
        return self.checked

class Button(Input):

    def __init__(self, name, *validators, **attrs):
        super(Button, self).__init__(name, *validators, **attrs)
        self.description = ""

    def render(self):
        attrs = self.attrs.copy()
        attrs['name'] = self.name
        if self.value is not None:
            attrs['value'] = self.value
        html = attrs.pop('html', None) or net.websafe(self.name)
        return '<button %s>%s</button>' % (attrs, html)

class Hidden(Input):

    def is_hidden(self):
        return True
        
    def get_type(self):
        return 'hidden'

class File(Input):

    def get_type(self):
        return 'file'
    
class Validator:
    def __deepcopy__(self, memo): return copy.copy(self)
    def __init__(self, msg, test, jstest=None): autoassign(self, locals())
    def valid(self, value): 
        try: return self.test(value)
        except: return False

notnull = Validator("Required", bool)

class regexp(Validator):
    def __init__(self, rexp, msg):
        self.rexp = re.compile(rexp)
        self.msg = msg
    
    def valid(self, value):
        return bool(self.rexp.match(value))

if __name__ == "__main__":
    import doctest
    doctest.testmod()



#!/usr/bin/env python
#coding:utf-8

import json
from bottle import MakoTemplate
from twisted.internet import reactor  
from twisted.internet.protocol import ReconnectingClientFactory
from autobahn.twisted.websocket import WebSocketClientProtocol, WebSocketClientFactory 
from toughradius.console.libs import utils

class WebSockProtocol(WebSocketClientProtocol):  

    messages = []
    callbacks = {}

    def onConnect(self, response):
        print("Radius Server connected: {0}".format(response.peer))

    def onOpen(self): 
        def send_message():
            if self.messages:
                self.sendMessage(self.messages.pop(),False) 
            self.factory.reactor.callLater(0.1, send_message) 
        send_message()

    def onMessage(self, msg, binary):  
        print "Got: " + msg  
        resp = json.loads(msg)
        if 'msg_id' in resp:
            callback = self.callbacks.get(resp['msg_id'])
            if callback and callable(callback):callback(resp)
            
    def onClose(self, wasClean, code, reason):
        print("WebSocket connection closed: {0}".format(reason))

class WSClientFactory(WebSocketClientFactory, ReconnectingClientFactory):

   protocol = WebSockProtocol

   def clientConnectionFailed(self, connector, reason):
      print("Client connection failed .. retrying ..")
      self.retry(connector)

   def clientConnectionLost(self, connector, reason):
      print("Client connection lost .. retrying ..")
      self.retry(connector)

class WebSock():    
    
    use_ssl = False

    def connect(self,radaddr,adminport): 
        def _connect_std():
            self.factory = WSClientFactory("ws://%s:%s"%(radaddr,adminport), debug = False)  
            reactor.connectTCP(radaddr, int(adminport), self.factory)
            
        if self.use_ssl:
            try:
                from twisted.internet import ssl
                contextFactory = ssl.ClientContextFactory()
                self.factory = WSClientFactory("wss://%s:%s"%(radaddr,adminport), debug = False)  
                reactor.connectSSL(radaddr, int(adminport), self.factory,contextFactory)
            except:
                _connect_std()
        else:
            _connect_std()
            

    def reconnect(self,radaddr,adminport):    
        self.connect(radaddr,adminport)

    def update_cache(self,cache_class,**kwargs):
        msg_id = utils.CurrentID()
        message = {
            'msg_id'  : msg_id,
            'process' : "admin_update_cache",
            'cache_class' : cache_class
        }
        callback = 'callback' in kwargs and kwargs.pop('callback') or None
        message.update(**kwargs)
        secure_msg = utils.encrypt(json.dumps(message).encode("utf-8"))
        self.factory.protocol.messages.append(secure_msg) 
        if callback:self.factory.protocol.callbacks[msg_id] = callback

    def invoke_admin(self,ops,**kwargs):
        msg_id = utils.CurrentID()
        message = {
            'msg_id'  : msg_id,
            'process' : "admin_%s"%ops
        }
        callback = 'callback' in kwargs and kwargs.pop('callback') or None
        message.update(**kwargs)
        secure_msg = utils.encrypt(json.dumps(message).encode("utf-8"))
        self.factory.protocol.messages.append(secure_msg) 
        if callback:self.factory.protocol.callbacks[msg_id] = callback

websock = WebSock()




#!/usr/bin/env python
#coding=utf-8
from twisted.python import log
from toughradius.radiusd.pyrad import packet
from toughradius.radiusd.settings import *
from toughradius.radiusd import utils
import logging
import datetime

def process(req=None,user=None,radiusd=None,**kwargs):
    if not req.get_acct_status_type() == STATUS_TYPE_START:
        return
    
    runstat=radiusd.runstat
    store = radiusd.store
    
    if store.is_online(req.get_nas_addr(),req.get_acct_sessionid()):
        runstat.acct_drop += 1
        return log.err('online %s is exists'%req.get_acct_sessionid())

    if not user:
        runstat.acct_drop += 1
        return log.err('user %s not exists'%req.get_user_name())

    runstat.acct_start += 1    
    online = utils.Storage(
        account_number = user['account_number'],
        nas_addr = req.get_nas_addr(),
        acct_session_id = req.get_acct_sessionid(),
        acct_start_time = datetime.datetime.now().strftime( "%Y-%m-%d %H:%M:%S"),
        framed_ipaddr = req.get_framed_ipaddr(),
        mac_addr = req.get_mac_addr(),
        nas_port_id = req.get_nas_portid(),
        billing_times = 0,
        input_total = 0,
        output_total = 0,
        start_source = STATUS_TYPE_START
    )

    store.add_online(online)

    log.msg('%s Accounting start request, add new online'%req.get_user_name(),level=logging.INFO)



#!/usr/bin/env python
#coding:utf-8
from __future__ import unicode_literals
import sys,os
from bottle import Bottle
from bottle import request
from bottle import response
from bottle import redirect
from bottle import MakoTemplate
from bottle import static_file
from bottle import abort
from beaker.cache import cache_managers
from toughradius.console.libs.paginator import Paginator
from toughradius.console.libs import utils
from toughradius.console.websock import websock
from toughradius.console import models
from toughradius.console.base import *
from toughradius.console.admin import forms
from hashlib import md5
import bottle
import datetime
import json
import functools

app = Bottle()
render = functools.partial(Render.render_app,app)

##############################################################################
# test handle
##############################################################################
@app.route('/test',apply=auth_opr)
def index(db):    
    form = forms.param_form()
    fparam = {}
    for p in db.query(models.SlcParam):
        fparam[p.param_name] = p.param_value
    form.fill(fparam)
    return render("base_form",form=form)

@app.get('/test/pid',apply=auth_opr)
def product_id(db):
    name = request.params.get("name")   
    product = db.query(models.SlcRadProduct).filter(
        models.SlcRadProduct.product_name == name
    ).first()
    return dict(pid=product.id)
    
@app.get('/test/mid',apply=auth_opr)
def member_id(db):
    name = request.params.get("name")   
    member = db.query(models.SlcMember).filter(
        models.SlcMember.member_name == name
    ).first()
    return dict(mid=member.member_id)
    
@app.route('/mksign',apply=auth_opr)
def index(db):    
    sign_args = request.params.get('sign_args')
    return dict(code=0,sign=utils.mk_sign(sign_args.strip().split(',')))
    
@app.post('/encrypt',apply=auth_opr)
def encrypt_data(db):    
    msg_data = request.params.get('data')
    return dict(code=0,data=utils.encrypt(msg_data))
    
@app.post('/decrypt',apply=auth_opr)
def decrypt_data(db):    
    msg_data = request.params.get('data')
    return dict(code=0,data=utils.decrypt(msg_data))
    
@app.get('/logquery/:name',apply=auth_opr)
def logquery(db,name):   
    def _query(logfile):
        if os.path.exists(logfile):
            with open(logfile) as f:
                f.seek(0,2)
                if f.tell() > 32*1024:
                    f.seek(f.tell()-32*1024)
                else:
                    f.seek(0)
                return f.read().replace('\n','<br>')
    if '%s.logfile'%name in app.config:
        logfile = app.config['%s.logfile'%name]
        return render("sys_logquery",msg=_query(logfile),title="%s logging"%name)
    else:
        return render("sys_logquery",msg="logfile not exists",title="%s logging"%name)
        
permit.add_route("/logquery/radiusd",u"radius系统日志查看",u"系统管理",is_menu=False,order=0.001,is_open=False)
permit.add_route("/logquery/admin",u"管理系统日志查看",u"系统管理",is_menu=False,order=0.001,is_open=False)
permit.add_route("/logquery/customer",u"自助系统日志查看",u"系统管理",is_menu=False,order=0.001,is_open=False)
###############################################################################
# Basic handle         
###############################################################################

@app.route('/',apply=auth_opr)
def index(db):    
    online_count = db.query(models.SlcRadOnline.id).count()
    user_total = db.query(models.SlcRadAccount.account_number).filter_by(status=1).count()
    return render("index",**locals())

@app.route('/static/:path#.+#')
def route_static(path):
    static_path = os.path.join(os.path.split(os.path.split(__file__)[0])[0],'static')
    return static_file(path, root=static_path)
    
###############################################################################
# update all cache      
###############################################################################    
@app.get('/cache/clean')
def clear_cache():
    def cbk(resp):
        print 'cbk',resp
    bottle.TEMPLATES.clear()
    for _cache in cache_managers.values():
        _cache.clear()
    websock.update_cache("all",callback=cbk)
    return dict(code=0,msg=u"已刷新缓存")
    
###############################################################################
# login handle         
###############################################################################

@app.get('/login')
def admin_login_get(db):
    return render("login")

@app.post('/login')
def admin_login_post(db):
    uname = request.forms.get("username")
    upass = request.forms.get("password")
    if not uname:return dict(code=1,msg=u"请填写用户名")
    if not upass:return dict(code=1,msg=u"请填写密码")
    enpasswd = md5(upass.encode()).hexdigest()
    opr = db.query(models.SlcOperator).filter_by(
        operator_name=uname,
        operator_pass=enpasswd
    ).first()
    if not opr:return dict(code=1,msg=u"用户名密码不符")
    if opr.operator_status == 1:return dict(code=1,msg=u"该操作员账号已被停用")
    set_cookie('username',uname)
    set_cookie('opr_type',opr.operator_type)
    set_cookie('login_time', utils.get_currtime())
    set_cookie('login_ip', request.remote_addr)  
    
    if opr.operator_type > 0:
        permit.unbind_opr(uname)
        for rule in db.query(models.SlcOperatorRule).filter_by(operator_name=uname):
            permit.bind_opr(rule.operator_name,rule.rule_path)  

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = uname
    ops_log.operate_ip = request.remote_addr
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)登陆'%(uname,)
    db.add(ops_log)
    db.commit()

    return dict(code=0,msg="ok")

@app.get("/logout")
def admin_logout(db):
    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)登出'%(get_cookie("username"),)
    db.add(ops_log)    
    db.commit()
    if get_cookie('opt_type') > 0:
        permit.unbind_opr(get_cookie("username"))
    set_cookie('username',None)
    set_cookie('login_time', None)
    set_cookie('opr_type',None)
    set_cookie('login_ip', None)   
    request.cookies.clear()
    redirect('/login')

###############################################################################
# param config      
############################################################################### 

@app.get('/param',apply=auth_opr)
def param(db):   
    form = forms.param_form()
    fparam = {}
    for p in db.query(models.SlcParam):
        fparam[p.param_name] = p.param_value
    form.fill(fparam)
    return render("sys_param",form=form)

@app.post('/param/update',apply=auth_opr)
def param_update(db): 
    params = db.query(models.SlcParam)
    for param in params:
        if param.param_name in request.forms:
            _value = request.forms.get(param.param_name)
            if _value: 
                param.param_value = _value  
                
    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)修改参数'%(get_cookie("username"))
    db.add(ops_log)
    db.commit()
    
    websock.reconnect(
        request.forms.get('radiusd_address'),
        request.forms.get('radiusd_admin_port'),
    )
        
    is_debug = request.forms.get('is_debug')
    bottle.debug(is_debug == '1')
    
    websock.update_cache("is_debug",is_debug=is_debug)
    websock.update_cache("reject_delay",reject_delay=request.forms.get('reject_delay'))
    websock.update_cache("param")
    redirect("/param")
    
permit.add_route("/param",u"系统参数管理",u"系统管理",is_menu=True,order=0)
permit.add_route("/param/update",u"系统参数修改",u"系统管理",is_menu=False,order=0,is_open=False)

###############################################################################
# password update     
###############################################################################

@app.get('/passwd',apply=auth_opr)
def passwd(db):   
    form=forms.passwd_update_form()
    form.fill(operator_name=get_cookie("username"))
    return render("base_form",form=form)

@app.post('/passwd',apply=auth_opr)
def passwd_update(db):  
    form=forms.passwd_update_form() 
    if not form.validates(source=request.forms):
        return render("base_form", form=form)
    if form.d.operator_pass != form.d.operator_pass_chk:
        return render("base_form", form=form,msg=u"确认密码不一致")
    opr = db.query(models.SlcOperator).first()
    opr.operator_pass = md5(form.d.operator_pass).hexdigest()

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)修改密码'%(get_cookie("username"),)
    db.add(ops_log)

    db.commit()
    redirect("/passwd")

###############################################################################
# node manage   
###############################################################################

@app.get('/node',apply=auth_opr)
def node(db):   
    return render("sys_node_list", page_data = get_page_data(db.query(models.SlcNode)))
    
permit.add_route("/node",u"区域信息管理",u"系统管理",is_menu=True,order=1)    

@app.get('/node/add',apply=auth_opr)
def node_add(db):  
    return render("base_form",form=forms.node_add_form())

@app.post('/node/add',apply=auth_opr)
def node_add_post(db): 
    form=forms.node_add_form()
    if not form.validates(source=request.forms):
        return render("base_form", form=form)
    node = models.SlcNode()
    node.node_name = form.d.node_name
    node.node_desc = form.d.node_desc
    db.add(node)

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)新增区域信息:%s'%(get_cookie("username"),node.node_name)
    db.add(ops_log)

    db.commit()
    redirect("/node")
    
permit.add_route("/node/add",u"新增区域",u"系统管理",order=1.01,is_open=False)

@app.get('/node/update',apply=auth_opr)
def node_update(db):  
    node_id = request.params.get("node_id")
    form=forms.node_update_form()
    form.fill(db.query(models.SlcNode).get(node_id))
    return render("base_form",form=form)

@app.post('/node/update',apply=auth_opr)
def node_add_update(db): 
    form=forms.node_update_form()
    if not form.validates(source=request.forms):
        return render("base_form", form=form)
    node = db.query(models.SlcNode).get(form.d.id)
    node.node_name = form.d.node_name
    node.node_desc = form.d.node_desc

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)修改区域信息:%s'%(get_cookie("username"),node.node_name)
    db.add(ops_log)

    db.commit()
    redirect("/node")    
    
permit.add_route("/node/update",u"修改区域",u"系统管理",order=1.02,is_open=False)

@app.get('/node/delete',apply=auth_opr)
def node_delete(db):     
    node_id = request.params.get("node_id")
    if db.query(models.SlcMember.member_id).filter_by(node_id=node_id).count()>0:
        return render("error",msg=u"该节点下有用户，不允许删除")
    db.query(models.SlcNode).filter_by(id=node_id).delete()

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)删除区域信息:%s'%(get_cookie("username"),node_id)
    db.add(ops_log)

    db.commit() 
    redirect("/node")  
    
permit.add_route("/node/delete",u"删除区域",u"系统管理",order=1.03,is_open=False)

###############################################################################
# bas manage    
###############################################################################

@app.route('/bas',apply=auth_opr,method=['GET','POST'])
def bas(db):   
    return render("sys_bas_list", 
        bastype = forms.bastype,
        bas_list = db.query(models.SlcRadBas))
        
permit.add_route("/bas",u"BAS信息管理",u"系统管理",is_menu=True,order=2)
    
@app.get('/bas/add',apply=auth_opr)
def bas_add(db):  
    return render("base_form",form=forms.bas_add_form())

@app.post('/bas/add',apply=auth_opr)
def bas_add_post(db): 
    form=forms.bas_add_form()
    if not form.validates(source=request.forms):
        return render("base_form", form=form)
    if db.query(models.SlcRadBas.id).filter_by(ip_addr=form.d.ip_addr).count()>0:
        return render("base_form", form=form,msg=u"Bas地址已经存在")        
    bas = models.SlcRadBas()
    bas.ip_addr = form.d.ip_addr
    bas.bas_name = form.d.bas_name
    bas.time_type = form.d.time_type
    bas.vendor_id = form.d.vendor_id
    bas.bas_secret = form.d.bas_secret
    bas.coa_port = form.d.coa_port
    db.add(bas)

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)新增BAS信息:%s'%(get_cookie("username"),bas.ip_addr)
    db.add(ops_log)

    db.commit()
    redirect("/bas")
    
permit.add_route("/bas/add",u"新增BAS",u"系统管理",order=2.01,is_open=False)

@app.get('/bas/update',apply=auth_opr)
def bas_update(db):  
    bas_id = request.params.get("bas_id")
    form=forms.bas_update_form()
    form.fill(db.query(models.SlcRadBas).get(bas_id))
    return render("base_form",form=form)

@app.post('/bas/update',apply=auth_opr)
def bas_add_update(db): 
    form=forms.bas_update_form()
    if not form.validates(source=request.forms):
        return render("base_form", form=form)
    bas = db.query(models.SlcRadBas).get(form.d.id)
    bas.bas_name = form.d.bas_name
    bas.time_type = form.d.time_type
    bas.vendor_id = form.d.vendor_id
    bas.bas_secret = form.d.bas_secret
    bas.coa_port = form.d.coa_port

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)修改BAS信息:%s'%(get_cookie("username"),bas.ip_addr)
    db.add(ops_log)

    db.commit()
    websock.update_cache("bas",ip_addr=bas.ip_addr)
    redirect("/bas")    
    
permit.add_route("/bas/update",u"修改BAS",u"系统管理",order=2.02,is_open=False)

@app.get('/bas/delete',apply=auth_opr)
def bas_delete(db):     
    bas_id = request.params.get("bas_id")
    db.query(models.SlcRadBas).filter_by(id=bas_id).delete()

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)删除BAS信息:%s'%(get_cookie("username"),bas_id)
    db.add(ops_log)

    db.commit() 
    redirect("/bas")    

permit.add_route("/bas/delete",u"删除BAS",u"系统管理",order=2.03,is_open=False)


###############################################################################
# opr manage    
###############################################################################

@app.route('/opr',apply=auth_opr,method=['GET','POST'])
def opr(db):   
    return render("sys_opr_list", 
        oprtype = forms.opr_type,
        oprstatus = forms.opr_status_dict,
        opr_list = db.query(models.SlcOperator))
        
permit.add_route("/opr",u"操作员管理",u"系统管理",is_menu=True,order=3,is_open=False)
    
@app.get('/opr/add',apply=auth_opr)
def opr_add(db):  
    nodes = [ (n.node_name,n.node_desc) for n in db.query(models.SlcNode)]
    form=forms.opr_add_form(nodes)
    return render("sys_opr_form",form=form,rules=[])

@app.post('/opr/add',apply=auth_opr)
def opr_add_post(db): 
    nodes = [ (n.node_name,n.node_desc) for n in db.query(models.SlcNode)]
    form=forms.opr_add_form(nodes)
    if not form.validates(source=request.forms):
        return render("sys_opr_form", form=form,rules=[])
    if db.query(models.SlcOperator.id).filter_by(operator_name=form.d.operator_name).count()>0:
        return render("sys_opr_form", form=form,rules=[],msg=u"操作员已经存在")   
        
    opr = models.SlcOperator()
    opr.operator_name = form.d.operator_name
    opr.operator_type = 1
    opr.operator_pass = md5(form.d.operator_pass).hexdigest()
    opr.operator_desc = form.d.operator_desc
    opr.operator_status = form.d.operator_status
    db.add(opr)
    
    for node in request.params.getall("operator_nodes"):
        onode = models.SlcOperatorNodes()
        onode.operator_name = form.d.operator_name
        onode.node_name = node
        db.add(onode)
        
    for path in request.params.getall("rule_item"):
        item = permit.get_route(path)
        if not item:continue
        rule = models.SlcOperatorRule()
        rule.operator_name = opr.operator_name
        rule.rule_name = item['name']
        rule.rule_path = item['path']
        rule.rule_category = item['category']
        db.add(rule)

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)新增操作员信息:%s'%(get_cookie("username"),opr.operator_name)
    db.add(ops_log)

    db.commit()
    redirect("/opr")
    
permit.add_route("/opr/add",u"新增操作员",u"系统管理",order=3.01,is_open=False)

@app.get('/opr/update',apply=auth_opr)
def opr_update(db):  
    opr_id = request.params.get("opr_id")
    opr = db.query(models.SlcOperator).get(opr_id)
    nodes = [ (n.node_name,n.node_desc) for n in db.query(models.SlcNode)]
    form=forms.opr_update_form(nodes)
    form.fill(opr)
    form.operator_pass.set_value('')
    onodes = db.query(models.SlcOperatorNodes).filter_by(operator_name=form.d.operator_name)
    form.operator_nodes.set_value([ond.node_name for ond in onodes])
    rules = db.query(models.SlcOperatorRule.rule_path).filter_by(operator_name=opr.operator_name)
    rules = [r[0] for r in rules]
    return render("sys_opr_form",form=form,rules=rules)

@app.post('/opr/update',apply=auth_opr)
def opr_add_update(db): 
    nodes = [ (n.node_name,n.node_desc) for n in db.query(models.SlcNode)]
    form=forms.opr_update_form(nodes)
    if not form.validates(source=request.forms):
        rules = db.query(models.SlcOperatorRule.rule_path).filter_by(operator_name=opr.operator_name)
        rules = [r[0] for r in rules]
        return render("sys_opr_form", form=form,rules=rules)
    opr = db.query(models.SlcOperator).get(form.d.id)
    
    if form.d.operator_pass:
        opr.operator_pass = md5(form.d.operator_pass).hexdigest()
    opr.operator_desc = form.d.operator_desc
    opr.operator_status = form.d.operator_status
    
    db.query(models.SlcOperatorNodes).filter_by(operator_name=opr.operator_name).delete()
    for node in request.params.getall("operator_nodes"):
        onode = models.SlcOperatorNodes()
        onode.operator_name = form.d.operator_name
        onode.node_name = node
        db.add(onode)
    
    # update rules
    db.query(models.SlcOperatorRule).filter_by(operator_name=opr.operator_name).delete()
    
    for path in request.params.getall("rule_item"):
        item = permit.get_route(path)
        if not item:continue
        rule = models.SlcOperatorRule()
        rule.operator_name = opr.operator_name
        rule.rule_name = item['name']
        rule.rule_path = item['path']
        rule.rule_category = item['category']
        db.add(rule)
        
    permit.unbind_opr(opr.operator_name)
    for rule in db.query(models.SlcOperatorRule).filter_by(operator_name=opr.operator_name):
        permit.bind_opr(rule.operator_name,rule.rule_path)

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)修改操作员信息:%s'%(get_cookie("username"),opr.operator_name)
    db.add(ops_log)

    db.commit()
    redirect("/opr")    
    
permit.add_route("/opr/update",u"修改操作员",u"系统管理",order=3.02,is_open=False)

@app.get('/opr/delete',apply=auth_opr)
def opr_delete(db):     
    opr_id = request.params.get("opr_id")
    opr = db.query(models.SlcOperator).get(opr_id)
    db.query(models.SlcOperatorNodes).filter_by(operator_name=opr.operator_name).delete()
    db.query(models.SlcOperatorRule).filter_by(operator_name=opr.operator_name).delete()
    db.query(models.SlcOperator).filter_by(id=opr_id).delete()

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)删除操作员信息:%s'%(get_cookie("username"),opr_id)
    db.add(ops_log)

    db.commit() 
    redirect("/opr")    

permit.add_route("/opr/delete",u"删除操作员",u"系统管理",order=3.03,is_open=False)



###############################################################################
# roster manage    
###############################################################################

@app.route('/roster',apply=auth_opr,method=['GET','POST'])
def roster(db):   
    _query = db.query(models.SlcRadRoster)
    return render("sys_roster_list", 
        page_data = get_page_data(_query))
        
permit.add_route("/roster",u"黑白名单管理",u"系统管理",is_menu=True,order=6)

@app.get('/roster/add',apply=auth_opr)
def roster_add(db):  
    return render("sys_roster_form",form=forms.roster_add_form())

@app.post('/roster/add',apply=auth_opr)
def roster_add_post(db): 
    form=forms.roster_add_form()
    if not form.validates(source=request.forms):
        return render("sys_roster_form", form=form)  
    if db.query(models.SlcRadRoster.id).filter_by(mac_addr=form.d.mac_addr).count()>0:
        return render("sys_roster_form", form=form,msg=u"MAC地址已经存在")     
    roster = models.SlcRadRoster()
    roster.mac_addr = form.d.mac_addr.replace("-",":").upper()
    roster.begin_time = form.d.begin_time
    roster.end_time = form.d.end_time
    roster.roster_type = form.d.roster_type
    db.add(roster)

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)新增黑白名单信息:%s'%(get_cookie("username"),roster.mac_addr)
    db.add(ops_log)

    db.commit()
    websock.update_cache("roster",mac_addr=roster.mac_addr)
    redirect("/roster")
    
permit.add_route("/roster/add",u"新增黑白名单",u"系统管理",order=6.01)    

@app.get('/roster/update',apply=auth_opr)
def roster_update(db):  
    roster_id = request.params.get("roster_id")
    form=forms.roster_update_form()
    form.fill(db.query(models.SlcRadRoster).get(roster_id))
    return render("sys_roster_form",form=form)

@app.post('/roster/update',apply=auth_opr)
def roster_add_update(db): 
    form=forms.roster_update_form()
    if not form.validates(source=request.forms):
        return render("sys_roster_form", form=form)       
    roster = db.query(models.SlcRadRoster).get(form.d.id)
    roster.begin_time = form.d.begin_time
    roster.end_time = form.d.end_time
    roster.roster_type = form.d.roster_type

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)修改黑白名单信息:%s'%(get_cookie("username"),roster.mac_addr)
    db.add(ops_log)

    db.commit()
    websock.update_cache("roster",mac_addr=roster.mac_addr)
    redirect("/roster")    
    
permit.add_route("/roster/update",u"修改白黑名单",u"系统管理",order=6.02)    

@app.get('/roster/delete',apply=auth_opr)
def roster_delete(db):     
    roster_id = request.params.get("roster_id")
    mac_addr = db.query(models.SlcRadRoster).get(roster_id).mac_addr
    db.query(models.SlcRadRoster).filter_by(id=roster_id).delete()

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)删除黑白名单信息:%s'%(get_cookie("username"),roster_id)
    db.add(ops_log)

    db.commit() 
    websock.update_cache("roster",mac_addr=mac_addr)
    redirect("/roster")        

permit.add_route("/roster/delete",u"删除黑白名单",u"系统管理",order=6.03)



#!/usr/bin/env python
#coding=utf-8
from toughradius.radiusd.plugins import error_auth
from toughradius.radiusd.settings import *
from toughradius.radiusd import utils

def process(req=None,resp=None,user=None,radiusd=None,**kwargs):
    """执行计费策略校验，用户到期检测，用户余额，时长检测"""
    store = radiusd.store
    acct_policy = user['product_policy'] or PPMonth
    if acct_policy in ( PPMonth,BOMonth):
        if utils.is_expire(user['expire_date']):
            resp['Framed-Pool'] = store.get_param("expire_addrpool")
            
    elif acct_policy in (PPTimes,PPFlow):
        user_balance = store.get_user_balance(user['account_number'])
        if user_balance <= 0:
            return error_auth(resp,'user balance poor')    
            
    elif acct_policy == BOTimes:
        time_length = store.get_user_time_length(user['account_number'])
        if time_length <= 0:
            return error_auth(resp,'user time_length poor')
            
    elif acct_policy == BOFlows:
        flow_length = store.get_user_flow_length(user['account_number'])
        if flow_length <= 0:
            return error_auth(resp,'user flow_length poor')

    if user['user_concur_number'] > 0 :
        if store.count_online(user['account_number']) >= user['user_concur_number']:
            return error_auth(resp,'user session to limit')    

    return resp


#!/usr/bin/env python
#coding:utf-8

from bottle import Bottle
from bottle import request
from bottle import response
from bottle import redirect
from bottle import static_file
from tablib import Dataset
from toughradius.console import models
from toughradius.console.admin import forms
from toughradius.console.websock import websock
from toughradius.console.libs import utils
from toughradius.console.libs.radius_attrs import radius_attrs
from toughradius.console.base import *
from sqlalchemy import func
import datetime
import bottle

__prefix__ = "/product"

app = Bottle()
app.config['__prefix__'] = __prefix__
render = functools.partial(Render.render_app,app)

###############################################################################
# product manage       
###############################################################################

@app.route('/',apply=auth_opr,method=['GET','POST'])
def product(db):   
    _query = db.query(models.SlcRadProduct)
    return render(
        "sys_product_list", 
        product_policys = forms.product_policy,
        node_list=db.query(models.SlcNode),
        page_data = get_page_data(_query)
    )
        
@app.get('/detail',apply=auth_opr)
def product_detail(db):
    product_id = request.params.get("product_id")   
    product = db.query(models.SlcRadProduct).get(product_id)
    if not product:
        return render("error",msg=u"资费不存在")
    product_attrs = db.query(models.SlcRadProductAttr).filter_by(product_id=product_id)
    return render("sys_product_detail",
        product_policys = forms.product_policy,
        product=product,product_attrs=product_attrs) 

permit.add_route(__prefix__,u"资费信息管理",u"系统管理",is_menu=True,order=4)
permit.add_route("%s/detail"%__prefix__,u"资费详情查看",u"系统管理",order=4.01)

@app.get('/add',apply=auth_opr)
def product_add(db):  
    return render("sys_product_form",form=forms.product_add_form())
    
@app.post('/add',apply=auth_opr)
def product_add_post(db): 
    form=forms.product_add_form()
    if not form.validates(source=request.forms):
        return render("sys_product_form", form=form)      
    product = models.SlcRadProduct()
    product.product_name = form.d.product_name
    product.product_policy = form.d.product_policy
    product.product_status = form.d.product_status
    product.fee_months = int(form.d.get("fee_months",0))
    product.fee_times = utils.hour2sec(form.d.get("fee_times",0))
    product.fee_flows = utils.mb2kb(form.d.get("fee_flows",0))
    product.bind_mac = form.d.bind_mac
    product.bind_vlan = form.d.bind_vlan
    product.concur_number = form.d.concur_number
    product.fee_period = form.d.fee_period
    product.fee_price = utils.yuan2fen(form.d.fee_price)
    product.input_max_limit = utils.mbps2bps(form.d.input_max_limit)
    product.output_max_limit = utils.mbps2bps(form.d.output_max_limit)
    _datetime = datetime.datetime.now().strftime( "%Y-%m-%d %H:%M:%S")
    product.create_time = _datetime
    product.update_time = _datetime
    db.add(product)

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)新增资费信息:%s'%(get_cookie("username"),product.product_name)
    db.add(ops_log)

    db.commit()
    redirect(__prefix__)
    
permit.add_route("%s/add"%__prefix__,u"新增资费",u"系统管理",order=4.02)

@app.get('/update',apply=auth_opr)
def product_update(db):  
    product_id = request.params.get("product_id")
    form=forms.product_update_form()
    product = db.query(models.SlcRadProduct).get(product_id)
    form.fill(product)
    form.product_policy_name.set_value(forms.product_policy[product.product_policy])
    form.fee_price.set_value(utils.fen2yuan(product.fee_price))
    return render("sys_product_form",form=form)

@app.post('/update',apply=auth_opr)
def product_update(db): 
    form=forms.product_update_form()
    if not form.validates(source=request.forms):
        return render("sys_product_form", form=form)
    product = db.query(models.SlcRadProduct).get(form.d.id)
    product.product_name = form.d.product_name
    product.product_status = form.d.product_status
    product.fee_months = int(form.d.get("fee_months",0))
    product.fee_times = int(form.d.get("fee_times",0))
    product.fee_flows = int(form.d.get("fee_flows",0))
    product.bind_mac = form.d.bind_mac
    product.bind_vlan = form.d.bind_vlan
    product.concur_number = form.d.concur_number
    product.fee_period = form.d.fee_period
    product.fee_price = utils.yuan2fen(form.d.fee_price)
    product.input_max_limit = utils.mbps2bps(form.d.input_max_limit) 
    product.output_max_limit = utils.mbps2bps(form.d.output_max_limit) 
    product.update_time = utils.get_currtime()

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)修改资费信息:%s'%(get_cookie("username"),product.product_name)
    db.add(ops_log)

    db.commit()
    websock.update_cache("product",product_id=product.id)
    redirect(__prefix__)    
    
permit.add_route("%s/update"%__prefix__,u"修改资费",u"系统管理",order=4.03)    

@app.get('/delete',apply=auth_opr)
def product_delete(db):     
    product_id = request.params.get("product_id")
    if db.query(models.SlcRadAccount).filter_by(product_id=product_id).count()>0:
        return render("error",msg=u"该套餐有用户使用，不允许删除") 
    if db.query(models.SlcRechargerCard).filter_by(product_id=product_id).count()>0:
        return render("error",msg=u"该套餐有发行充值卡，不允许删除")   
        
    db.query(models.SlcRadProduct).filter_by(id=product_id).delete()

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)删除资费信息:%s'%(get_cookie("username"),product_id)
    db.add(ops_log)

    db.commit() 
    websock.update_cache("product",product_id=product_id)
    redirect(__prefix__)   
    
permit.add_route("%s/delete"%__prefix__,u"删除资费",u"系统管理",order=4.04)

@app.get('/attr/add',apply=auth_opr)
def product_attr_add(db): 
    product_id = request.params.get("product_id")
    if db.query(models.SlcRadProduct).filter_by(id=product_id).count()<=0:
        return render("error",msg=u"资费不存在") 
    form = forms.product_attr_add_form()
    form.product_id.set_value(product_id)
    return render("sys_pattr_form",form=form,pattrs=radius_attrs)

@app.post('/attr/add',apply=auth_opr)
def product_attr_add(db): 
    form = forms.product_attr_add_form()
    if not form.validates(source=request.forms):
        return render("sys_pattr_form", form=form,pattrs=radius_attrs)   
    attr = models.SlcRadProductAttr()
    attr.product_id = form.d.product_id
    attr.attr_name = form.d.attr_name
    attr.attr_value = form.d.attr_value
    attr.attr_desc = form.d.attr_desc
    db.add(attr)

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)新增资费属性信息:%s'%(get_cookie("username"),attr.attr_name)
    db.add(ops_log)

    db.commit()

    redirect("%s/detail?product_id=%s"%(__prefix__,form.d.product_id)) 
    
permit.add_route("%s/attr/add"%__prefix__,u"新增资费扩展属性",u"系统管理",order=4.05)

@app.get('/attr/update',apply=auth_opr)
def product_attr_update(db): 
    attr_id = request.params.get("attr_id")
    attr = db.query(models.SlcRadProductAttr).get(attr_id)
    form = forms.product_attr_update_form()
    form.fill(attr)
    return render("sys_pattr_form",form=form,pattrs=radius_attrs)

@app.post('/attr/update',apply=auth_opr)
def product_attr_update(db): 
    form = forms.product_attr_update_form()
    if not form.validates(source=request.forms):
        return render("pattr_form", form=form,pattrs=radius_attrs)   
    attr = db.query(models.SlcRadProductAttr).get(form.d.id)
    attr.attr_name = form.d.attr_name
    attr.attr_value = form.d.attr_value
    attr.attr_desc = form.d.attr_desc

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)修改资费属性信息:%s'%(get_cookie("username"),attr.attr_name)
    db.add(ops_log)

    db.commit()
    websock.update_cache("product",product_id=form.d.product_id)
    redirect("%s/detail?product_id=%s"%(__prefix__,form.d.product_id))
    
permit.add_route("%s/attr/update"%__prefix__,u"修改资费扩展属性",u"系统管理",order=4.06)

@app.get('/attr/delete',apply=auth_opr)
def product_attr_update(db): 
    attr_id = request.params.get("attr_id")
    attr = db.query(models.SlcRadProductAttr).get(attr_id)
    product_id = attr.product_id
    db.query(models.SlcRadProductAttr).filter_by(id=attr_id).delete()

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)删除资费属性信息:%s'%(get_cookie("username"),serial_json(attr))
    db.add(ops_log)

    db.commit()
    websock.update_cache("product",product_id=product_id)
    redirect("%s/detail?product_id=%s"%(__prefix__,product_id))
    
permit.add_route("%s/attr/delete"%__prefix__,u"删除资费扩展属性",u"系统管理",order=4.07)